/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.util.ArrayList;
import java.util.List;

public class DemoPagableTable extends DTable {

	private static final long serialVersionUID = -7006130816222005690L;
	private int allRows = 22;
	private int allColumns = 40;

	public DemoPagableTable() {
		// setFormattingLocale("sl", "SI");
		setFrozenTableEnabled(true);
		setPageableTableEnabled(true);
		setHighlightActiveRowEnabled(true);
		setWordWrapInHeaderCellsEnabled(false);
		setWordWrapInBodyCellsEnaled(false);
		setHorizontalLinesEnabled(false);
		setAllColumnsEnabled(false);
		setFilterHelperEnabled(true);
		setFixedTableHeightEnabled(true);
		setHorizontalLinesEnabled(false);
		setVerticalLinesEnabled(true);
		setLoadingMessageEnabled(true);
		setLoadingMessageDelay(100, 300);

		setExpandedFrozenHeaderCellEnabled(true);
		setExpandedFrozenHeaderCellForActiveColumnEnabled(true);
		setExpandedFrozenBodyCellEnabled(true);
		setSortedFrozenTableEnabled(false);
		setFilteredFrozenTableEnabled(true);
		setHideFrozenTableEnabled(true);
		setResizeFrozenTableEnabled(true);
		setEmptyFrozenTableEnabled(true);
		setUnfreezeAnyColumnsEnabled(true);
		setUnfreezeColumnsEnabled(true, 3);
		setFrozenTableMinimalWidth(200);

		setExpandedHeaderCellEnabled(true);
		setExpandedHeaderCellForActiveColumnEnabled(true);
		setExpandedBodyCellEnabled(true);
		setSortedTableEnabled(true);
		setFilteredTableEnabled(true);
		setFreezeAnyColumnsEnabled(true);

		setColumnFrozens(1, 2);
		setUnfreezeColumnsEnabled(true, 1);
		// setColumnHiddens(3, 6);
		setColumnDefaultWidth(120);
		setHeaderRowHeight(24);
		setBodyRowHeight(22);
		setPageSize(15);
		setPagerPages(15);
		setDateTimeFormats("yyyy-MM-dd HH:mm:ss", "dd.MM.yyyy HH:mm:ss", 3);
		setNumberFormats("#0.00", 8, 9);
		setNumberFormats("#,##0.00", 1, 2, 5);
		setColumnWidths(45, 73, 133, 166, 66, 33); //
		//
		setHeaderAlignments(HorizontalAlignment.LEFT, HorizontalAlignment.LEFT,
				HorizontalAlignment.LEFT, HorizontalAlignment.LEFT,
				HorizontalAlignment.LEFT, HorizontalAlignment.LEFT,
				HorizontalAlignment.LEFT);
		setColumnAlignment(0, HorizontalAlignment.CENTER);
		//
		List<String> headers = new ArrayList<String>();
		for (int i = 1; i <= allColumns; i++) {
			headers.add("HEADER " + i);
		}
		setColumnHeaders(headers.toArray(new String[] {}));
		for (int i = 500; i <= 2000; i++) {
			// setColumnHiddens(i);
		}
	}

	@Override
	public Object[] getRowsData(int startRow, int rows, Sorting[] sortings,
			Filter[] filters) {
		rows = rows == 0 ? allRows : rows;

		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows)
				&& row < getAllRows(filters); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 0) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 3 || column == 10) {
					data.add("2009-03-31 06:24:08");
				} else {
					data.add((float) row * 100 + (float) column / 100);
				}
			}
		}
		//
		/*
		 * if (sortings != null && sortings.length > 0) { String string = new
		 * String(); for (Sorting sorting : sortings) { string += "-- " +
		 * sorting.getColumn() + " - " + sorting.getValue() + " --" + " - " +
		 * sorting.getType() + " --"; } //
		 * getApplication().getMainWindow().showNotification( "Sorting: " + //
		 * string, Notification.TYPE_ERROR_MESSAGE); }
		 */
		//
		return (filters != null && filters.length > 1 ? null : data.toArray());
	}

	@Override
	public long getAllRows(Filter[] filters) {
		if (filters != null && filters.length > 2) {
			return 0;
		}
		return (filters == null || filters.length == 0 ? allRows : allRows
				/ (filters.length + 1));
	}

}
