/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.util.ArrayList;
import java.util.List;

public class DemoScrolableTable extends DTable {

	private static final long serialVersionUID = -6865664679839888794L;
	private int allRows = 115;
	private int allColumns = 88;

	public DemoScrolableTable() {
		// setFormattingLocale("sl", "SI");
		setFrozenTableEnabled(true);
		setScrollableTableEnabled(true);
		setHighlightActiveRowEnabled(true);
		setWordWrapInHeaderCellsEnabled(false);
		setWordWrapInBodyCellsEnaled(false);
		setHorizontalLinesEnabled(false);
		setAllColumnsEnabled(false);
		setFilterHelperEnabled(true);
		setFixedTableHeightEnabled(true);
		setHorizontalLinesEnabled(false);
		setVerticalLinesEnabled(true);
		setLoadingMessageEnabled(true);
		setLoadingMessageOpacity(0.5f);
		setLoadingMessageDelay(100, 300);

		setExpandedFrozenHeaderCellEnabled(true);
		setExpandedFrozenBodyCellEnabled(true);
		setSortedFrozenTableEnabled(false);
		setFilteredFrozenTableEnabled(true);
		setHideFrozenTableEnabled(true);
		setResizeFrozenTableEnabled(true);
		setEmptyFrozenTableEnabled(true);
		setUnfreezeAnyColumnsEnabled(true);
		setUnfreezeColumnsEnabled(true, 3);

		setExpandedHeaderCellEnabled(true);
		setExpandedBodyCellEnabled(true);
		setSortedTableEnabled(true);
		setFilteredTableEnabled(true);
		setFreezeAnyColumnsEnabled(true);

		setFrozenTableMinimalWidth(200);
		setTableHeight(300);
		setHeaderRowHeight(24);
		setBodyRowHeight(22);
		setColumnFrozens(0, 2, 4);
		// enableUnfreezeColumns(false, 7, 33);
		// enableUnfreezeColumns(true, 11);
		setColumnDefaultWidth(70);
		// setHeaderRowHeight(22);
		setPagerPages(15);

		setDateTimeFormats("yyyy-MM-dd HH:mm:ss", "dd.MM.yyyy HH:mm:ss", 10);
		setStandardDateTimeFormats("yyyy-MM-dd HH:mm:ss",
				DateTimeFormat.DATE_MEDIUM, 9);
		setNumberFormats("#0.00", 8);
		setNumberFormats("#,##0.00", 0, 2, 5);
		setColumnWidths(140, 273, 133, 166, 166, 133, 142, 73, 73, 33);
		// frozen setColumnWidths(140, 3133, 166); //
		// non frozen setColumnWidths(273, 1166, 2133, 342, 73, 73, 33); //
		setColumnAlignments(HorizontalAlignment.CENTER,
				HorizontalAlignment.CENTER, HorizontalAlignment.CENTER,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT,
				HorizontalAlignment.LEFT, HorizontalAlignment.RIGHT);
		setHeaderDefaultAlignment(HorizontalAlignment.CENTER);
		setColumnSortings(Sorting.ASCEDING, 1, 6);
		setColumnSortings(Sorting.DESCEDING, 4, 5);

		List<String> headers = new ArrayList<String>();
		for (int i = 1; i <= allColumns; i++) {
			headers.add("HEADER " + i);
		}

		setColumnHeaders(headers.toArray(new String[] {}));
		for (int i = 100; i <= 600; i += 2) {
			// setColumnHiddens(i);
		}
	}

	@Override
	public Object[] getRowsData(int startRow, int rows, Sorting[] sortings,
			Filter[] filters) {
		rows = rows == 0 ? allRows : rows;

		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows)
				&& row < getAllRows(filters); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 1) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 9 || column == 10) {
					data.add("2009-03-31 06:24:08");
				} else {
					if (row % 5 == 2) {
						data.add(null);
					} else {
						data.add((float) row + (float) column / 100);
					}
				}
			}
		}
		//
		/*
		 * if (sortings != null && sortings.length > 0) { String string = new
		 * String(); for (Sorting sorting : sortings) { string += "-- " +
		 * sorting.getColumn() + " - " + sorting.getValue() + " --" + " - " +
		 * sorting.getType() + " --"; } //
		 * getApplication().getMainWindow().showNotification( "Sorting: " + //
		 * string, Notification.TYPE_ERROR_MESSAGE); }
		 */
		//
		return data.toArray();
	}

	@Override
	public long getAllRows(Filter[] filters) {
		if (filters != null && filters.length > 2) {
			return 0;
		}
		return (filters == null || filters.length == 0 ? allRows : allRows
				/ (filters.length + 1));
	}

}
