/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.io.Serializable;

public class Sorting implements Serializable {

	private static final long serialVersionUID = -3531022478168363659L;
	public static final String ASCEDING = "+";
	public static final String DESCEDING = "-";
	//
	private String column;
	private String header;
	private String headerId;

	public Sorting(String column, String header, String headerId) {
		if (column != null && column.length() > 0) {
			if (!ASCEDING.equals(column.substring(0, 1))
					&& !DESCEDING.equals(column.substring(0, 1))) {
				this.column = ASCEDING + column;
			} else {
				this.column = column;
			}
			this.header = header;
			this.headerId = headerId;
		} else {
			this.column = null;
		}
	}

	public Integer getColumn() {
		if (column != null && column.length() >= 2) {
			return Math.abs(Integer.parseInt(column.substring(1)));
		}
		return null;
	}

	public String getValue() {
		return column;
	}

	public String getHeader() {
		return header;
	}

	public String getHeaderId() {
		return headerId;
	}

	public String getType() {
		if (column != null && column.length() >= 2) {
			return column.substring(0, 1);
		}
		return null;
	}

	@Override
	public String toString() {
		return column;
	}

	@Override
	public boolean equals(Object obj) {
		if (column.equals(obj)) {
			return true;
		}
		return false;
	}

}
