/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.ui;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.GQuery.window;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.google.gwt.dom.client.Style.Cursor;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.query.client.css.RGBColor;
import com.google.gwt.query.client.css.TakesCssValue.CssSetter;
import com.google.gwt.query.client.css.TextAlignProperty.TextAlign;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.SimplePanel;
import com.vaadin.terminal.gwt.client.ApplicationConnection;
import com.vaadin.terminal.gwt.client.BrowserInfo;
import com.vaadin.terminal.gwt.client.Paintable;
import com.vaadin.terminal.gwt.client.UIDL;

public class VDTable extends SimplePanel implements Paintable {

	private static final TextAlign[] ALIGN = { TextAlign.LEFT, TextAlign.RIGHT,
			TextAlign.CENTER };

	private Map<String, Object> request = new HashMap<String, Object>();
	private List<Integer> columnAlignments = new ArrayList<Integer>();
	private List<Integer> columnWidths = new ArrayList<Integer>();
	private List<Integer> columnFormatted = new ArrayList<Integer>();
	private List<Integer> columnFrozens = new ArrayList<Integer>();
	private List<Integer> columnHiddens = new ArrayList<Integer>();
	private List<Integer> columnFreezeEnabled = new ArrayList<Integer>();
	private List<Integer> columnUnfreezeEnabled = new ArrayList<Integer>();
	private Map<Integer, List<Object>> columnFilters = new HashMap<Integer, List<Object>>();

	private int browserOffset = 0;
	private int tableFrozenMinimalWidth = 0;
	private int tableFrozenWidth = 0;
	private int tableMinimalWidth = 0;
	private int tableHeight = 0;
	private int headerRowHeight = 24;
	private int bodyRowHeight = 22;
	private int pageSize = 0;
	private int activeRows = 1;
	private int firstRow = 0;
	private int allRows = 0;
	private int mouseX = 0;
	private int splitterLeft = 0;
	private int windowX = 0;
	private int windowY = 0;
	private int scrollLeftFrozen = 0;
	private int scrollLeft = 0;
	private int scrollTop = 0;
	private int controledScroll = 0;
	private int ffMarginTop = 0;
	private int loadingMessageShowDelay = 100;
	private int loadingMessageHideDelay = 300;
	private float loadingMessageOpacity = 0.5f;
	private String loadingMessage = "Loading data, please wait...";
	private String emptyDataMessage = "No data to display!";

	private boolean enableFixedTableHeight = false;
	private boolean enableAllColumns = false;
	private boolean enableAllRows = false;
	private boolean enablePageableTable = false;
	private boolean enableFrozenTable = false;
	private boolean enableHideFrozenTable = false;
	private boolean enableResizeFrozenTable = false;
	private boolean enableEmptyFrozenTable = false;
	private boolean enableFreezeAnyColumns = false;
	private boolean enableUnfreezeAnyColumns = false;
	private boolean enableSortedFrozenTable = false;
	private boolean enableSortedTable = false;
	private boolean enableFilteredFrozenTable = false;
	private boolean enableFilteredTable = false;
	private boolean enableExpandedFrozenHeaderCell = false;
	private boolean enableExpandedFrozenHeaderCellForActiveColumn = false;
	private boolean enableExpandedFrozenBodyCell = false;
	private boolean enableExpandedHeaderCell = false;
	private boolean enableExpandedHeaderCellForActiveColumn = false;
	private boolean enableExpandedBodyCell = false;
	private boolean enableWordWrapInHeaderCells = false;
	private boolean enableWordWrapInBodyCells = false;
	private boolean enableFilterHelper = true;
	private boolean enableFilterHelperOnTop = false;
	private boolean enableHorizontalLines = true;
	private boolean enableVerticalLines = true;
	private boolean enableLoadingMessage = false;

	private int id;
	protected String uidl;
	protected ApplicationConnection client;
	private VMenu menu = null;
	private VPager pager = null;

	private boolean informServer = true;
	private Boolean fetchNewData = null;
	private boolean allColumnsVisible = false;
	private boolean allRowsVisible = false;
	private boolean enableResizeTimer = false;
	private boolean enableScrollTimer = false;
	private boolean showFrozenTable = false;
	private boolean showSplitter = false;

	private GQuery splitterClickEventHandler = null;
	private GQuery splitterMoveEventHandler = null;
	private GQuery tableResizeEventHandler = null;

	private GQuery frozenTableScrollEventHandler = null;
	private GQuery frozenHeaderCellHoverEventHandler = null;
	private GQuery frozenHeaderCellClickEventHandler = null;
	private GQuery frozenHeaderCellContextMenuEventHandler = null;
	private GQuery frozenBodyRowHighlightEventHandler = null;
	private GQuery frozenBodyCellHoverEventHandler = null;
	private GQuery frozenBodyCellContextMenuEventHandler = null;

	private GQuery tableScrollEventHandler = null;
	private GQuery headerCellHoverEventHandler = null;
	private GQuery headerCellClickEventHandler = null;
	private GQuery headerCellContextMenuEventHandler = null;
	private GQuery bodyRowHighlightEventHandler = null;
	private GQuery bodyCellHoverEventHandler = null;
	private GQuery bodyCellContextMenuEventHandler = null;

	public VDTable() {
		// global id
		id = (int) (Math.random() * 1000000);

		getElement().setId("v-dtable" + id);
		setStyleName("v-dtable");
	}

	protected int getId() {
		return id;
	}

	private void appendFrozenTable() {
		// header section
		Element frozenHeaderWrapper = DOM.createTD();
		frozenHeaderWrapper.setId("dt-frozen-header-wrapper");

		Element frozenHeaderContainer = DOM.createDiv();
		frozenHeaderContainer.setId("dt-frozen-header-container");
		frozenHeaderWrapper.appendChild(frozenHeaderContainer);

		Element frozenHeaderTable = DOM.createTable();
		frozenHeaderTable.setId("dt-frozen-header-table" + getId());
		frozenHeaderTable.getStyle().setWidth(100, Unit.PCT);
		frozenHeaderContainer.appendChild(frozenHeaderTable);

		Element splitterContainer = DOM.createTD();
		splitterContainer.setId("dt-splitter-container");
		splitterContainer.setAttribute("rowspan", "2");
		splitterContainer.getStyle().setCursor(Cursor.DEFAULT);

		$("table#dt-header-table" + getId()).parent().parent()
				.before($(frozenHeaderWrapper));
		$("table#dt-header-table" + getId()).parent().parent()
				.before($(splitterContainer));

		// body section
		Element frozenBodyWrapper = DOM.createTD();
		frozenBodyWrapper.setId("dt-frozen-body-wrapper");

		Element frozenBodyContainer = DOM.createDiv();
		frozenBodyContainer.setId("dt-frozen-body-container");
		frozenBodyWrapper.appendChild(frozenBodyContainer);

		Element frozenBodyTable = DOM.createTable();
		frozenBodyTable.setId("dt-frozen-body-table" + getId());
		frozenBodyTable.getStyle().setWidth(100, Unit.PCT);
		frozenBodyContainer.appendChild(frozenBodyTable);

		Element frozenBodyContainerLength = DOM.createDiv();
		frozenBodyContainerLength.setId("dt-frozen-body-container-length"
				+ getId());
		frozenBodyContainerLength
				.setClassName("dt-frozen-body-container-length");
		frozenBodyContainer.appendChild(frozenBodyContainerLength);

		$("table#dt-body-table" + getId()).parent().parent()
				.before($(frozenBodyWrapper));
	}

	private void initContainers() {
		browserOffset = BrowserInfo.get().isChrome() ? 0 : 1;

		Element mainContainerWrapper = DOM.createDiv();
		mainContainerWrapper.setId("dt-main-container-wrapper");
		getElement().appendChild(mainContainerWrapper);

		Element mainContainer = DOM.createTable();
		mainContainer.setId("dt-main-container");
		mainContainerWrapper.appendChild(mainContainer);

		// header section
		Element trHeader = DOM.createTR();
		mainContainer.appendChild(trHeader);

		Element headerWrapper = DOM.createTD();
		headerWrapper.setId("dt-header-wrapper");
		trHeader.appendChild(headerWrapper);

		Element headerContainer = DOM.createDiv();
		headerContainer.setId("dt-header-container");
		headerWrapper.appendChild(headerContainer);

		Element headerTable = DOM.createTable();
		headerTable.setId("dt-header-table" + getId());
		headerTable.getStyle().setWidth(100, Unit.PCT);
		headerContainer.appendChild(headerTable);

		Element headerWrapperEmpty = DOM.createTD();
		headerWrapperEmpty.setId("dt-header-wrapper-empty");
		trHeader.appendChild(headerWrapperEmpty);

		// body section
		Element trBody = DOM.createTR();
		mainContainer.appendChild(trBody);

		Element bodyWrapper = DOM.createTD();
		bodyWrapper.setId("dt-body-wrapper");
		bodyWrapper.setPropertyInt("colSpan", 2);
		trBody.appendChild(bodyWrapper);

		Element bodyContainer = DOM.createDiv();
		bodyContainer.setId("dt-body-container");
		bodyWrapper.appendChild(bodyContainer);

		Element bodyTable = DOM.createTable();
		bodyTable.setId("dt-body-table" + getId());
		bodyTable.getStyle().setWidth(100, Unit.PCT);
		bodyContainer.appendChild(bodyTable);

		Element bodyContainerLength = DOM.createDiv();
		bodyContainerLength.setId("dt-body-container-length" + getId());
		bodyContainerLength.setClassName("dt-body-container-length");
		bodyContainer.appendChild(bodyContainerLength);

		if (menu == null) {
			menu = new VMenu(this);
			menu.setGlassEnabled(true);
		}
	}

	private void activateSplitterClickEventHandler(boolean enable) {
		if (enable) {
			final GQuery splitterContainer = $("div#v-dtable" + getId()).find(
					"td#dt-splitter-container");
			final GQuery mainContainerWrapper = $("div#v-dtable" + getId())
					.find("div#dt-main-container-wrapper");
			final GQuery frozenBodyWrapper = $(
					"table#dt-frozen-body-table" + getId()).parent().parent();

			activateSplitterClickEventHandler(false);
			activateSplitterMoveEventHandler(false);
			splitterClickEventHandler = mainContainerWrapper.delegate("td",
					"mousedown", new Function() {
						public boolean f(Event event) {
							if ("dt-splitter-container".equals(getElement()
									.getId())) {
								if (frozenBodyWrapper.width() == 0) {
									showFrozenTable(true);
									if (isLastRowVisible()
											&& !allColumnsVisible) {
										sendMessage("newData",
												Boolean.FALSE.toString(), true);
									} else {
										setContainerWidthAndHeight();
									}
									return false;
								} else if (enableResizeFrozenTable) {
									mouseX = event.getClientX();
									splitterLeft = splitterContainer.position().left
											- browserOffset;
									CssSetter left = CSS.LEFT.with(Length.px(0));
									CssSetter top = CSS.TOP.with(Length.px(0));
									CssSetter width = CSS.WIDTH.with(Length
											.px(mainContainerWrapper.width()
													+ browserOffset));
									CssSetter height = CSS.HEIGHT.with(Length
											.px(mainContainerWrapper.height()
													+ browserOffset));
									CssSetter sLeft = CSS.LEFT.with(Length
											.px(splitterLeft));
									CssSetter sTop = CSS.TOP.with(Length.px(0));
									CssSetter sWidth = CSS.WIDTH.with(Length
											.px(splitterContainer.width()));
									CssSetter sHeight = CSS.HEIGHT.with(Length
											.px(splitterContainer.height()));
									mainContainerWrapper
											.append($("<div/>")
													.id("dt-splitter-background")
													.html("&nbsp;")
													.css(CSS.CURSOR
															.with(Cursor.COL_RESIZE))
													.css(left).css(top)
													.css(width).css(height))
											.append($("<div/>")
													.id("dt-splitter")
													.html("&nbsp;").css(sLeft)
													.css(sTop).css(sWidth)
													.css(sHeight));
									activateSplitterMoveEventHandler(true);
									return false;
								} else if (enableHideFrozenTable) {
									showFrozenTable(false);
									if (isLastRowVisible()
											&& !allColumnsVisible) {
										sendMessage("newData",
												Boolean.FALSE.toString(), true);
									} else {
										setContainerWidthAndHeight();
									}
									return false;
								}
							}
							return true;
						}
					});
		} else if (!enable && splitterClickEventHandler != null) {
			splitterClickEventHandler.undelegate("td", "mousedown");
			splitterClickEventHandler = null;
		}
	}

	private void activateSplitterMoveEventHandler(boolean enable) {
		if (enable) {
			final String splitter = "#dt-splitter";
			final String splitterBackground = "dt-splitter-background";
			final String[][] filter = {
					{ "td#dt-frozen-header-wrapper",
							"td#dt-frozen-body-wrapper" },
					{ "div#dt-frozen-header-container",
							"div#dt-frozen-body-container" } };
			final GQuery table = $("div#v-dtable" + getId());
			final GQuery mainContainerWrapper = $("div#v-dtable" + getId())
					.find("div#dt-main-container-wrapper");

			activateSplitterMoveEventHandler(false);
			splitterMoveEventHandler = table
					.delegate("div", "mousemove", new Function() {
						public void f(Element e) {
							if (splitterBackground.equals(e.getId())) {
								int left = splitterLeft
										+ getEvent().getClientX() - mouseX;
								int right = Math.min(left,
										mainContainerWrapper.width()
												- tableMinimalWidth);
								left = Math.max(tableFrozenMinimalWidth, right);
								$(splitter).css(CSS.LEFT.with(Length.px(left)));
							}
						}
					}).delegate("div", "mouseup", new Function() {
						public void f(Element e) {
							if (splitterBackground.equals(e.getId())) {
								int xMouse = getEvent().getClientX();
								if (enableHideFrozenTable && xMouse == mouseX) {
									showFrozenTable(false);
									if (isLastRowVisible()
											&& !allColumnsVisible) {
										sendMessage("newData",
												Boolean.FALSE.toString(), true);
									} else {
										setContainerWidthAndHeight();
									}
								} else {
									int left = splitterLeft + xMouse - mouseX;
									int right = Math.min(left,
											mainContainerWrapper.width()
													- tableMinimalWidth);
									left = Math.max(tableFrozenMinimalWidth,
											right);
									mainContainerWrapper
											.children("table")
											.children("tr")
											.children(filter[0])
											.css(CSS.WIDTH.with(Length.px(left)));
									mainContainerWrapper
											.children("table")
											.children("tr")
											.children("td")
											.children(filter[1])
											.css(CSS.WIDTH.with(Length.px(left)));
									tableFrozenWidth = left;
								}

								$(e).remove();
								$(splitter).remove();
								activateSplitterMoveEventHandler(false);

								// set width to 100% - Chrome want this
								mainContainerWrapper.children("tr")
										.children("td").children(filter[1])
										.css(CSS.WIDTH.with(Length.pct(100)));

								setContainerWidthAndHeight();
								if (isLastRowVisible() && !allRowsVisible) {
									sendMessage("newData",
											Boolean.TRUE.toString(), true);
								}
							}
						}
					}).delegate("div", "mouseout", new Function() {
						public void f(Element e) {
							if (splitterBackground.equals(e.getId())) {
								$(e).remove();
								$(splitter).remove();
								activateSplitterMoveEventHandler(false);
							}
						}
					});
		} else if (!enable && splitterMoveEventHandler != null) {
			splitterMoveEventHandler.undelegate("div", "mousemove")
					.undelegate("div", "mouseup").undelegate("div", "mouseout");
			splitterMoveEventHandler = null;
		}
	}

	private void activateTableResizeEventHandler(boolean enable) {
		if (enable) {
			final GQuery containerLength = $("div#dt-body-container-length"
					+ getId());
			final GQuery header = $("table#dt-header-table" + getId());

			activateTableResizeEventHandler(false);
			tableResizeEventHandler = $(window).resize(new Function() {
				public void f() {
					containerLength.css(CSS.WIDTH.with(Length.px(header.width())));
					if (!enableResizeTimer) {
						updateTableOnResizing();
					}
				};
			});
		} else if (!enable && tableResizeEventHandler != null) {
			tableResizeEventHandler.die();
			tableResizeEventHandler = null;
		}
	}

	private void activateFrozenTableScrollEventHandler(boolean enable) {
		if (enable) {
			final GQuery frozenHeader = $(
					"table#dt-frozen-header-table" + getId()).parent();
			final GQuery frozenBody = $("table#dt-frozen-body-table" + getId())
					.parent();

			activateFrozenTableScrollEventHandler(false);
			frozenTableScrollEventHandler = frozenBody.scroll(new Function() {
				public void f() {
					scrollLeftFrozen = frozenBody.scrollLeft();
					frozenHeader.scrollLeft(scrollLeftFrozen);
				};
			});
		} else if (!enable && frozenTableScrollEventHandler != null) {
			frozenTableScrollEventHandler.die();
			frozenTableScrollEventHandler = null;
		}
	}

	private void activateFrozenHeaderCellHoverEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-frozen-header-table" + getId());

			activateFrozenHeaderCellHoverEventHandler(false);
			frozenHeaderCellHoverEventHandler = header.delegate("td",
					"mouseover", new Function() {
						public void f(Element e) {
							GQuery eDiv = $(e).children("div");
							int width = eDiv.width();
							int scrollWidth = e.getFirstChildElement()
									.getScrollWidth();

							if (width < scrollWidth) {
								eDiv.addClass("expanded");
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public void f(Element e) {
					$(e).children("div.expanded").removeClass("expanded");
				}
			});
		} else if (!enable && frozenHeaderCellHoverEventHandler != null) {
			frozenHeaderCellHoverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			frozenHeaderCellHoverEventHandler = null;
		}
	}

	private void activateFrozenHeaderCellClickEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-frozen-header-table" + getId());

			activateFrozenHeaderCellClickEventHandler(false);
			frozenHeaderCellClickEventHandler = header.delegate("td", "click",
					new Function() {
						public void f(Element e) {
							if (enableSortedFrozenTable) {
								String clazz = $(e).hasClass("asc") ? "desc"
										: "asc";
								$(e).parent().children("td.asc", "td.desc")
										.removeClass("asc", "desc");
								$(e).addClass(clazz);
								sendMessage("sortings", getSortings(), true);
							}
						}
					});
		} else if (!enable && frozenHeaderCellClickEventHandler != null) {
			frozenHeaderCellClickEventHandler.undelegate("td", "click");
			frozenHeaderCellClickEventHandler = null;
		}
	}

	private void activateFrozenHeaderCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-frozen-header-table" + getId());

			activateFrozenHeaderCellContextMenuEventHandler(false);
			frozenHeaderCellContextMenuEventHandler = header.delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event event) {
							menu.add((Element) getElement());
							menu.setPopupPosition(DOM.eventGetClientX(event),
									DOM.eventGetClientY(event));
							menu.show();
							return false;
						}
					});
		} else if (!enable && frozenHeaderCellContextMenuEventHandler != null) {
			frozenHeaderCellContextMenuEventHandler.undelegate("td",
					"contextmenu");
			frozenHeaderCellContextMenuEventHandler = null;
		}
	}

	private void activateFrozenBodyRowHighlightEventHandler(boolean enable) {
		if (enable) {
			final GQuery frozenBody = $("table#dt-frozen-body-table" + getId());
			final GQuery body = $("table#dt-body-table" + getId());

			activateFrozenBodyRowHighlightEventHandler(false);
			frozenBodyRowHighlightEventHandler = frozenBody.delegate("tr",
					"mouseover", new Function() {
						public void f(Element e) {
							if (!$(e).hasClass("empty")
									&& frozenBody.children("tr").size() > 1) {
								int counter = Integer
										.parseInt($(e).children("td").eq(0)
												.id().split(":")[1]);

								if ($(e).parent().children("tr").eq(0)
										.hasClass("empty")) {
									counter += 1;
								}

								$(e).addClass("hover");
								body.children("tr").eq(counter)
										.addClass("hover");
							}
						}
					}).delegate("tr", "mouseout", new Function() {
				public void f(Element e) {
					if (!$(e).hasClass("empty")
							&& frozenBody.children("tr").size() > 1) {
						int counter = Integer.parseInt($(e).children("td")
								.eq(0).id().split(":")[1]);

						if ($(e).parent().children("tr").eq(0)
								.hasClass("empty")) {
							counter += 1;
						}

						$(e).removeClass("hover");
						body.children("tr").eq(counter).removeClass("hover");
					}
				}
			});
		} else if (!enable && frozenBodyRowHighlightEventHandler != null) {
			frozenBodyRowHighlightEventHandler.undelegate("tr", "mouseover")
					.undelegate("tr", "mouseout");
			frozenBodyRowHighlightEventHandler = null;
		}
	}

	private void activateFrozenBodyCellHoverEventHandler(boolean enable) {
		if (enable) {
			GQuery body = $("table#dt-frozen-body-table" + getId());
			final String headerTd = "table#dt-frozen-header-table" + getId()
					+ " tr td";

			activateFrozenBodyCellHoverEventHandler(false);
			frozenBodyCellHoverEventHandler = body.delegate("td", "mouseover",
					new Function() {
						public void f(Element e) {
							final String[] eId = e.getId().split(":");

							if (eId != null && eId.length == 3) {
								GQuery eDiv = $(e).children("div");
								int width = eDiv.width();
								int scrollWidth = e.getFirstChildElement()
										.getScrollWidth();

								if (width < scrollWidth) {
									eDiv.addClass("expanded");
									if (bodyRowHighlightEventHandler == null) {
										eDiv.css(
												"background-color",
												eDiv.parent()
														.parent()
														.css("background-color"));
									}
								}
							}

							// checking header expanded cell
							if (enableExpandedFrozenHeaderCellForActiveColumn
									&& eId != null && eId.length == 3) {
								$(headerTd).each(new Function() {
									public void f(Element e) {
										if ($(e).id().equals(
												eId[0] + ":" + eId[2])) {
											GQuery eDiv = $(e).children("div");
											int width = eDiv.width();
											int scrollWidth = e
													.getFirstChildElement()
													.getScrollWidth();

											if (width < scrollWidth) {
												eDiv.addClass("expanded");
											}
										}
									}
								});
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public void f(Element e) {
					$(e).children("div").removeClass("expanded")
							.css("background-color", "");
					$(headerTd).children("div.expanded")
							.removeClass("expanded");
				};
			});
		} else if (!enable && frozenBodyCellHoverEventHandler != null) {
			frozenBodyCellHoverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			frozenBodyCellHoverEventHandler = null;
		}
	}

	private void activateFrozenBodyCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			GQuery body = $("table#dt-frozen-body-table" + getId());

			activateFrozenBodyCellContextMenuEventHandler(false);
			frozenBodyCellContextMenuEventHandler = body.delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event event) {
							if (enableFilteredFrozenTable) {
								if (!((Element) getElement()).getInnerHTML()
										.contains("<img ")) {
									menu.add((Element) getElement());
									menu.setPopupPosition(
											DOM.eventGetClientX(event),
											DOM.eventGetClientY(event));
									menu.show();
								}
							}
							return false;
						}
					});
		} else if (!enable && frozenBodyCellContextMenuEventHandler != null) {
			frozenBodyCellContextMenuEventHandler.undelegate("td",
					"contextmenu");
			frozenBodyCellContextMenuEventHandler = null;
		}
	}

	private void activateTableScrollEventHandler(boolean enable) {
		if (enable) {
			final GQuery header = $("table#dt-header-table" + getId()).parent();
			final GQuery body = $("table#dt-body-table" + getId()).parent();
			final GQuery frozenBody = $("table#dt-frozen-body-table" + getId());

			activateTableScrollEventHandler(false);
			tableScrollEventHandler = body.scroll(new Function() {
				public void f() {
					header.scrollLeft(body.scrollLeft());
					if (BrowserInfo.get().isFirefox()) {
						// FF bug: flickering if vertical scrollbar is hidden
						frozenBody.css(CSS.MARGIN_TOP.with(Length.px(-body
								.scrollTop() + ffMarginTop)));
					} else {
						frozenBody.parent().scrollTop(body.scrollTop());
					}

					if (!enableScrollTimer) {
						updateTableOnScrolling();
					}
				};
			});
		} else if (!enable && tableScrollEventHandler != null) {
			tableScrollEventHandler.die();
			tableScrollEventHandler = null;
		}
	}

	private void activateHeaderCellHoverEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-header-table" + getId());

			activateHeaderCellHoverEventHandler(false);
			headerCellHoverEventHandler = header.delegate("td", "mouseover",
					new Function() {
						public void f(Element e) {
							GQuery eDiv = $(e).children("div");
							int width = eDiv.width();
							int scrollWidth = e.getFirstChildElement()
									.getScrollWidth();

							if (width < scrollWidth) {
								eDiv.addClass("expanded");
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public void f(Element e) {
					$(e).children("div.expanded").removeClass("expanded");
				}
			});
		} else if (!enable && headerCellHoverEventHandler != null) {
			headerCellHoverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			headerCellHoverEventHandler = null;
		}
	}

	private void activateHeaderCellClickEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-header-table" + getId());

			activateHeaderCellClickEventHandler(false);
			headerCellHoverEventHandler = header.delegate("td", "click",
					new Function() {
						public void f(Element e) {
							if (enableSortedTable) {
								String clazz = $(e).hasClass("asc") ? "desc"
										: "asc";
								$(e).parent().children("td.asc", "td.desc")
										.removeClass("asc", "desc");
								$(e).addClass(clazz);
								sendMessage("sortings", getSortings(), true);
							}
						}
					});
		} else if (!enable && headerCellClickEventHandler != null) {
			headerCellClickEventHandler.undelegate("td", "click");
			headerCellClickEventHandler = null;
		}
	}

	private void activateHeaderCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			GQuery header = $("table#dt-header-table" + getId());

			activateHeaderCellContextMenuEventHandler(false);
			headerCellContextMenuEventHandler = header.delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event event) {
							menu.add((Element) getElement());
							menu.setPopupPosition(DOM.eventGetClientX(event),
									DOM.eventGetClientY(event));
							menu.show();
							return false;
						}
					});
		} else if (!enable && headerCellContextMenuEventHandler != null) {
			headerCellContextMenuEventHandler.undelegate("td", "contextmenu");
			headerCellContextMenuEventHandler = null;
		}
	}

	private void activateBodyRowHighlightEventHandler(boolean enable) {
		if (enable) {
			final GQuery body = $("table#dt-body-table" + getId());
			final GQuery frozenBody = $("table#dt-frozen-body-table" + getId());

			activateBodyRowHighlightEventHandler(false);
			bodyRowHighlightEventHandler = body.delegate("tr", "mouseover",
					new Function() {
						public void f(Element e) {
							if (!$(e).hasClass("empty")
									&& body.children("tr").size() > 1) {
								int counter = Integer
										.parseInt($(e).children("td").eq(0)
												.id().split(":")[1]);

								if ($(e).parent().children("tr").eq(0)
										.hasClass("empty")) {
									counter += 1;
								}

								$(e).addClass("hover");
								frozenBody.children("tr").eq(counter)
										.addClass("hover");
							}
						}
					}).delegate("tr", "mouseout", new Function() {
				public void f(Element e) {
					if (!$(e).hasClass("empty")
							&& body.children("tr").size() > 1) {
						int counter = Integer.parseInt($(e).children("td")
								.eq(0).id().split(":")[1]);

						if ($(e).parent().children("tr").eq(0)
								.hasClass("empty")) {
							counter += 1;
						}

						$(e).removeClass("hover");
						frozenBody.children("tr").eq(counter)
								.removeClass("hover");
					}
				}
			});
		} else if (!enable && bodyRowHighlightEventHandler != null) {
			bodyRowHighlightEventHandler.undelegate("tr", "mouseover")
					.undelegate("tr", "mouseout");
			bodyRowHighlightEventHandler = null;
		}
	}

	private void activateBodyCellHoverEventHandler(boolean enable) {
		if (enable) {
			GQuery body = $("table#dt-body-table" + getId());
			final String headerTd = "table#dt-header-table" + getId()
					+ " tr td";

			activateBodyCellHoverEventHandler(false);
			bodyCellHoverEventHandler = body.delegate("td", "mouseover",
					new Function() {
						public void f(Element e) {
							final String[] eId = e.getId().split(":");

							if (eId != null && eId.length == 3) {
								GQuery eDiv = $(e).children("div");
								int width = eDiv.width();
								int scrollWidth = e.getFirstChildElement()
										.getScrollWidth();

								if (width < scrollWidth) {
									eDiv.addClass("expanded");
									if (bodyRowHighlightEventHandler == null) {
										eDiv.css(
												"background-color",
												eDiv.parent()
														.parent()
														.css("background-color"));
									}
								}
							}

							// checking header expanded cell
							if (enableExpandedHeaderCellForActiveColumn
									&& eId != null && eId.length == 3) {
								$(headerTd).each(new Function() {
									public void f(Element e) {
										if ($(e).id().equals(
												eId[0] + ":" + eId[2])) {
											GQuery eDiv = $(e).children("div");
											int width = eDiv.width();
											int scrollWidth = e
													.getFirstChildElement()
													.getScrollWidth();

											if (width < scrollWidth) {
												eDiv.addClass("expanded");
											}
										}
									}
								});
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public void f(Element e) {
					$(e).children("div").removeClass("expanded")
							.css("background-color", "");
					$(headerTd).children("div.expanded")
							.removeClass("expanded");
				};
			});
		} else if (!enable && bodyCellHoverEventHandler != null) {
			bodyCellHoverEventHandler.undelegate("td", "mouseover").undelegate(
					"td", "mouseout");
			bodyCellHoverEventHandler = null;
		}
	}

	private void activateBodyCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			GQuery body = $("table#dt-body-table" + getId());

			activateBodyCellContextMenuEventHandler(false);
			bodyCellContextMenuEventHandler = body.delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event event) {
							if (enableFilteredTable) {
								if (!((Element) getElement()).getInnerHTML()
										.contains("<img ")) {
									menu.add((Element) getElement());
									menu.setPopupPosition(
											DOM.eventGetClientX(event),
											DOM.eventGetClientY(event));
									menu.show();
								}
							}
							return false;
						}
					});
		} else if (!enable && bodyCellContextMenuEventHandler != null) {
			bodyCellContextMenuEventHandler.undelegate("td", "contextmenu");
			bodyCellContextMenuEventHandler = null;
		}
	}

	private void updateTableOnResizing() {
		if (windowX != Window.getClientWidth()
				|| windowY != Window.getClientHeight()) {
			enableResizeTimer = true;

			Timer timer = new Timer() {
				public void run() {
					int winX = Window.getClientWidth();
					int winY = Window.getClientHeight();

					if (windowX == winX && windowY == winY) {
						// check if table width is smaller then
						// table default minimal width
						if (enableFrozenTable && showFrozenTable) {
							GQuery body = $("table#dt-body-table" + getId())
									.parent();
							if (body.outerWidth() < tableMinimalWidth) {
								String[][] filter = {
										{ "td#dt-frozen-header-wrapper",
												"td#dt-frozen-body-wrapper" },
										{ "div#dt-frozen-header-container",
												"div#dt-frozen-body-container" } };
								GQuery mainContainerWrapper = $(
										"div#v-dtable" + getId()).find(
										"div#dt-main-container-wrapper");
								mainContainerWrapper
										.children("table")
										.children("tr")
										.children(filter[0])
										.css(CSS.WIDTH.with(Length
												.px(tableFrozenMinimalWidth)));
								mainContainerWrapper
										.children("table")
										.children("tr")
										.children("td")
										.children(filter[1])
										.css(CSS.WIDTH.with(Length
												.px(tableFrozenMinimalWidth)));
								setContainerWidthAndHeight();
							}
						}

						cancel();
						enableResizeTimer = false;
						setContainerWidthAndHeight();
					}

					windowX = Window.getClientWidth();
					windowY = Window.getClientHeight();
				}
			};

			timer.scheduleRepeating(500);
		}
	}

	private void updateTableOnScrolling() {
		final GQuery body = $("table#dt-body-table" + getId()).parent();

		if (scrollLeft != body.scrollLeft() || scrollTop != body.scrollTop()) {
			enableScrollTimer = true;

			Timer timer = new Timer() {
				public void run() {
					int left = body.scrollLeft();
					int top = body.scrollTop();

					if (scrollLeft == left && scrollTop == top) {
						if (!enableAllRows || !enableAllColumns) {
							fetchNewData = scrollLeft == controledScroll;
							controledScroll = scrollLeft;
							if ((fetchNewData && !allRowsVisible)
									|| (!fetchNewData && !allColumnsVisible)) {
								sendMessage("newData", fetchNewData.toString(),
										true);
							}
						}

						cancel();
						enableScrollTimer = false;
						fetchNewData = null;
					}

					scrollLeft = left;
					scrollTop = top;
				}
			};

			timer.scheduleRepeating(600);
		}
	}

	protected int getAllColumns(boolean tableFrozen) {
		if (tableFrozen) {
			return columnFrozens.size();
		}
		return (columnAlignments.size() - columnFrozens.size());
	}

	private boolean isLastRowVisible() {
		return ((firstRow + activeRows) == allRows);
	}

	private boolean isHidden(int column) {
		return columnHiddens.contains(column);
	}

	protected boolean isFrozenTableEnabled() {
		return enableFrozenTable;
	}

	private void appendLoadingMessageBox(boolean append) {
		if (append) {
			appendLoadingMessageBox(false);
			GQuery message = $("<div/>")
					.id("dt-message")
					.addClass("dt-message")
					.css(CSS.DISPLAY.with(Display.NONE))
					.append($("<div/>")
							.id("dt-message-wrapper")
							.addClass("wrapper")
							.css(CSS.DISPLAY.with(Display.NONE))
							.append($("<div/>").text(loadingMessage).addClass(
									"context")));
			$("div#v-dtable" + getId()).find("table#dt-main-container").after(
					message);
		} else {
			$("div#dt-message").remove();
		}
	}

	private void showLoadingMessage(boolean show) {
		GQuery wrapper = $("div#v-dtable" + getId()).find(
				"div#dt-main-container-wrapper");
		GQuery message = $("div#v-dtable" + getId() + " div#dt-message");
		if (show) {
			message.css("display", "").children("div").css("display", "")
					.parent().css(CSS.LEFT.with(Length.px(0)))
					.css(CSS.TOP.with(Length.px(0)))
					.css(CSS.WIDTH.with(Length.px(wrapper.width())))
					.css(CSS.HEIGHT.with(Length.px(wrapper.height())))
					.css(CSS.ZINDEX.with(10000)).as(Effects)
					.animate("opacity:1", loadingMessageShowDelay);
		} else {
			message.as(Effects).animate("opacity:0", loadingMessageHideDelay,
					new Function() {
						public void f(Element e) {
							$(e).css(CSS.DISPLAY.with(Display.NONE))
									.css(CSS.LEFT.with(Length.px(0)))
									.css(CSS.TOP.with(Length.px(0)))
									.css(CSS.WIDTH.with(Length.px(0)))
									.css(CSS.HEIGHT.with(Length.px(0)))
									.css(CSS.ZINDEX.with(-1));
						}
					});
			wrapper.children("table").children("tr").as(Effects)
					.animate("opacity:1", loadingMessageHideDelay);
		}
	}

	private void showFrozenTable(boolean show) {
		final String[][] filter = {
				{ "td#dt-frozen-header-wrapper", "td#dt-frozen-body-wrapper" },
				{ "div#dt-frozen-header-container",
						"div#dt-frozen-body-container" } };
		GQuery mainContainer = $("div#v-dtable" + getId()
				+ " table#dt-main-container");

		if (show && !showFrozenTable) {
			showFrozenTable = show;
			mainContainer.children("tr").children(filter[0])
					.css(CSS.WIDTH.with(Length.px(tableFrozenWidth)));
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.px(tableFrozenWidth)));

			// set width to 100% - Chrome want this
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.pct(100)));
		} else if (!show && showFrozenTable) {
			showFrozenTable = show;
			mainContainer.children("tr").children(filter[0])
					.css(CSS.WIDTH.with(Length.px(0)));
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.px(0)));
		}
	}

	private void showSplitter(boolean show) {
		final String[][] filter = {
				{ "td#dt-frozen-header-wrapper", "td#dt-frozen-body-wrapper" },
				{ "div#dt-frozen-header-container",
						"div#dt-frozen-body-container" } };
		GQuery mainContainer = $("div#v-dtable" + getId()
				+ " table#dt-main-container");

		if (show && !showSplitter) {
			showSplitter = show;
			showFrozenTable = show;
			mainContainer.children("tr").children("td#dt-splitter-container")
					.css("width", "").css("display", "");
			mainContainer.children("tr").children(filter[0])
					.css(CSS.WIDTH.with(Length.px(tableFrozenWidth)));
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.px(tableFrozenWidth)));

			// set width to 100% - Chrome want this
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.pct(100)));
		} else if (!show && showSplitter) {
			showSplitter = show;
			showFrozenTable = show;
			mainContainer.children("tr").children("td#dt-splitter-container")
					.css(CSS.WIDTH.with(Length.px(0)))
					.css(CSS.DISPLAY.with(Display.NONE));
			mainContainer.children("tr").children(filter[0])
					.css(CSS.WIDTH.with(Length.px(0)));
			mainContainer.children("tr").children("td").children(filter[1])
					.css(CSS.WIDTH.with(Length.px(0)));
		}
	}

	protected boolean isSortingsEnabled(int column) {
		return ((allRows > 1) && (!isFrozen(column) ? enableSortedTable
				: enableSortedFrozenTable));
	}

	protected String getSortings() {
		final StringBuilder builder = new StringBuilder();

		GQuery header = $("table#dt-header-table" + getId() + " tr");
		header.children("td.asc").each(new Function() {
			public void f(Element e) {
				String id = e.getId().split(":")[1];
				builder.append("+" + id).append("_:_");
			}
		});
		header.children("td.desc").each(new Function() {
			public void f(Element e) {
				String id = e.getId().split(":")[1];
				builder.append("-" + id).append("_:_");
			}
		});

		if (enableFrozenTable) {
			GQuery frozenHeader = $("table#dt-frozen-header-table" + getId()
					+ " tr");
			frozenHeader.children("td.asc").each(new Function() {
				public void f(Element e) {
					String id = e.getId().split(":")[1];
					builder.append("+" + id).append("_:_");
				}
			});
			frozenHeader.children("td.desc").each(new Function() {
				public void f(Element e) {
					String id = e.getId().split(":")[1];
					builder.append("-" + id).append("_:_");
				}
			});
		}

		return (builder.length() == 0 ? "+1-" : builder.substring(0,
				builder.length() - 3));
	}

	protected boolean isFrozenTableEmptyEnabled() {
		return enableEmptyFrozenTable;
	}

	protected boolean isFreezeAnyColumnsEnabled() {
		return enableFreezeAnyColumns;
	}

	protected boolean isUnfreezeAnyColumnsEnabled() {
		return enableUnfreezeAnyColumns;
	}

	protected boolean isFreezeEnabled(int column) {
		if (columnFreezeEnabled.contains(column)) {
			return true;
		} else if (columnFreezeEnabled.contains(-column)) {
			return false;
		} else if (!columnFreezeEnabled.contains(column)
				&& !columnFreezeEnabled.contains(-column)
				&& !enableFreezeAnyColumns) {
			return false;
		}
		return true;
	}

	protected boolean isUnfreezeEnabled(int column) {
		if (columnUnfreezeEnabled.contains(column)) {
			return true;
		} else if (columnUnfreezeEnabled.contains(-column)) {
			return false;
		} else if (!columnUnfreezeEnabled.contains(column)
				&& !columnUnfreezeEnabled.contains(-column)
				&& !enableUnfreezeAnyColumns) {
			return false;
		}
		return true;
	}

	protected boolean isFrozen(int column) {
		return columnFrozens.contains(column);
	}

	protected void setFrozen(int column) {
		if ((!enableFilteredFrozenTable && !isFrozen(column) && isFiltered(column))
				|| (!enableFilteredTable && isFrozen(column) && isFiltered(column))) {
			request.put("removeFilter", Integer.toString(column));
		}

		if (!enableSortedFrozenTable && !isFrozen(column)) {
			$("table#dt-header-table" + getId()).children("tr")
					.children("td [id='" + getId() + ":" + column + "']")
					.removeClass("asc", "desc");
			request.put("sortings", getSortings());
		} else if (!enableSortedTable && isFrozen(column)) {
			$("table#dt-frozen-header-table" + getId()).children("tr")
					.children("td [id='" + getId() + ":" + column + "']")
					.removeClass("asc", "desc");
			request.put("sortings", getSortings());
		}

		columnFrozens.clear();
		sendMessage("freeze", Integer.toString(column), true);
	}

	protected boolean isFormatted(int column) {
		return columnFormatted.contains(column);
	}

	protected boolean isFilterEnabled(int column) {
		return (allRows > 1 && (!isFrozen(column) ? enableFilteredTable
				: enableFilteredFrozenTable));
	}

	protected boolean isFiltered(int column) {
		return columnFilters.containsKey(column);
	}

	protected boolean filterOperatorExists(int column, String operator) {
		if (isFiltered(column)) {
			List<Object> filters = columnFilters.get(column);
			for (Object filter : filters) {
				if (((String[]) filter)[0].equals(operator)) {
					return true;
				}
			}
		}
		return false;
	}

	protected boolean filterExists(int column, String filter) {
		if (isFiltered(column)) {
			List<Object> filters = columnFilters.get(column);
			for (Object tempFilter : filters) {
				if (((String[]) tempFilter)[1].equals(filter)) {
					return true;
				}
			}
		}
		return false;
	}

	protected void newFilter(int column, int row, String operator,
			String description, String filter) {
		if (operator != null && !operator.isEmpty()) {
			sendMessage("newFilter", (Integer.toString(column) + "_:_"
					+ Integer.toString(row) + "_:_" + operator + "_:_"
					+ description + "_:_" + filter), true);
		}
	}

	protected void removeFilters(int column) {
		if (isFiltered(column)) {
			sendMessage("removeFilter", Integer.toString(column), true);
		}
	}

	private void setFilterHtml() {
		// String filterString = new String("FILTERS: ");
		String filterHtml = new String();
		for (Entry<Integer, List<Object>> columnFilter : columnFilters
				.entrySet()) {
			String html = new String();
			String headerId = "table#dt-"
					+ (isFrozen(columnFilter.getKey()) ? "frozen-" : "")
					+ "header-table" + getId() + " tr";
			String header = $(headerId).children(
					"td [id='" + getId() + ":" + columnFilter.getKey() + "']")
					.text();

			for (Object filter : columnFilter.getValue()) {
				if (html.isEmpty()) {
					html = "<b>[" + header + "]:</b> ";
					html += ((String[]) filter)[1];
				} else {
					html += ", " + ((String[]) filter)[1];
				}
			}

			filterHtml += html + "; ";
		}
		filterHtml.trim();

		$("#v-dtable" + getId() + " div#dt-filter-helper").remove();
		if (enableFilterHelper && !filterHtml.isEmpty()) {
			GQuery mainContainerWrapper = $("#v-dtable" + getId()).find(
					"div#dt-main-container-wrapper");
			filterHtml = "<b><u>FILTRI:</u></b> " + filterHtml;

			if (enableFilterHelperOnTop) {
				mainContainerWrapper.before($("<div/>").id("dt-filter-helper")
						.addClass("dt-filter-helper").html(filterHtml));
			} else {
				mainContainerWrapper.after($("<div/>").id("dt-filter-helper")
						.addClass("dt-filter-helper").html(filterHtml));
			}
		}
	}

	private void setFilter(int column, String description, String filter,
			boolean addColumnFilter) {
		if (addColumnFilter) {
			if (filter != null && filter.length() > 0) {
				List<Object> filters = columnFilters.get(column);
				if (filters == null) {
					filters = new ArrayList<Object>();
				}
				filters.add(new String[] { description, filter });
				columnFilters.put(column, filters);
			}
		} else {
			columnFilters.remove(column);
		}

		setFilterHtml();
	}

	private void setColumnSorting(int column, String sorting) {
		GQuery td = $("td [id='" + getId() + ":" + column + "']");

		td.removeClass("asc", "desc");
		if ("asc".equals(sorting)) {
			td.addClass("asc");
		} else if ("desc".equals(sorting)) {
			td.addClass("desc");
		}
	}

	private GQuery createBaseTd(CssSetter align, CssSetter width) {
		GQuery baseTd = $("<td/>").css(align).css(width);
		$("<span/>").appendTo($("<div/>").appendTo(baseTd));
		return baseTd;
	}

	private GQuery createBaseTr(GQuery body, int[] columns, int left,
			boolean checkWidth) {
		if (columns != null && columns.length > 0) {
			int screenWidth = BrowserInfo.get().getScreenWidth();
			int controledWidth = -body.scrollLeft() + left;
			GQuery baseTr = $("<tr/>");

			for (int column : columns) {
				if (!isHidden(column)) {
					CssSetter align = CSS.TEXT_ALIGN
							.with(ALIGN[columnAlignments.get(column)]);
					CssSetter width = CSS.WIDTH.with(Length.px(columnWidths
							.get(column) - browserOffset));
					createBaseTd(align, width).appendTo(baseTr);

					if (checkWidth) {
						controledWidth += columnWidths.get(column);
						if (controledWidth > screenWidth) {
							break;
						}
					}
				}
			}

			if (!enableHorizontalLines) {
				baseTr.children("td").css(
						CSS.BORDER_TOP_COLOR.with(RGBColor.TRANSPARENT));
				baseTr.children("td").css(
						CSS.BORDER_BOTTOM_COLOR.with(RGBColor.TRANSPARENT));
			}

			if (!enableVerticalLines) {
				baseTr.children("td").css(
						CSS.BORDER_LEFT_COLOR.with(RGBColor.TRANSPARENT));
				baseTr.children("td").css(
						CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
			}

			return baseTr;
		}
		return null;
	}

	private GQuery createEmptyTr(GQuery tr, int height, String trClass,
			String divClass) {
		CssSetter cssHeight = CSS.LINE_HEIGHT.with(Length.px(height));
		return tr.clone().addClass(trClass).children("td").children("div")
				.css(cssHeight).addClass(divClass).children("span")
				.html("&nbsp;").parent().parent().parent();
	}

	private GQuery setTrValues(GQuery tr, final String id,
			final List<String> values, final int[] columns) {
		return tr.children("td").each(new Function() {
			public Object f(Element e, int col) {
				return setTdValue($(e), id + columns[col], values.get(col));
			}
		}).parent();
	}

	private GQuery setTdValue(GQuery td, String id, String value) {
		if (value == null || value.isEmpty()) {
			return td.id(id).children("div").children("span").html("&nbsp;");
		} else if (value.startsWith("image_:_")) {
			return setTdImage(td, id, value);
		}
		return td.id(id).children("div").children("span").text(value);
	}

	private GQuery setTdImage(GQuery td, String id, String value) {
		String[] data = value.split("_:_");
		int width = Integer.parseInt(data[2]);
		int height = Integer.parseInt(data[3]);
		int top = 0;
		int bottom = 0;
		float zoom = (float) (bodyRowHeight - 1) / height;

		zoom = zoom > 1 ? 1 : zoom;
		width *= zoom;
		height *= zoom;
		top = (bodyRowHeight - 1 - height) / 2;
		bottom = bodyRowHeight - 1 - height - top;

		Element image = DOM.createImg();
		image.setPropertyString("src", data[1]);
		image.setPropertyString("width", Integer.toString(width));
		image.setPropertyString("height", Integer.toString(height));
		image.getStyle().setMarginTop(top, Unit.PX);
		image.getStyle().setMarginBottom(bottom, Unit.PX);

		return td.id(id).children("div").children("span").replaceWith(image);
	}

	private void setFrozenHeader(int[] columns) {
		if (columns != null) {
			columnFrozens.clear();

			GQuery frozenHeader = $("table#dt-frozen-header-table" + getId()
					+ " tr");
			GQuery header = $("table#dt-header-table" + getId() + " tr");

			for (int column : columns) {
				columnFrozens.add(column);
				GQuery cell = header.children("td [id='" + getId() + ":"
						+ column + "']");
				frozenHeader.append(cell.clone());
				cell.remove();
			}
		}
	}

	private void setHeader(String[] headers, int[] alignments,
			int tableFrozenWidth) {
		GQuery frozenHeader = $("table#dt-frozen-header-table" + getId());
		GQuery header = $("table#dt-header-table" + getId());
		CssSetter height = CSS.LINE_HEIGHT.with(Length.px(headerRowHeight - 1));
		String clazz = enableWordWrapInHeaderCells ? "normal" : "nowrap";

		frozenHeader.empty().append($("<tr/>"));
		header.empty().append($("<tr/>"));

		for (int column = 0; column < headers.length; column++) {
			if (!isHidden(column)) {
				String tdId = getId() + ":" + column;
				CssSetter align = CSS.TEXT_ALIGN
						.with(ALIGN[alignments[column]]);
				CssSetter width = CSS.WIDTH.with(Length.px(columnWidths
						.get(column) - browserOffset));
				$("<div/>")
						.css(height)
						.addClass(clazz)
						.append($("<span/>").text(headers[column]))
						.appendTo(
								$("<td/>").id(tdId).css(align).css(width)
										.appendTo(header.children("tr")));
			}
		}
		frozenHeader.parent().parent()
				.css(CSS.WIDTH.with((Length.px(tableFrozenWidth))));

		if (!enableVerticalLines) {
			frozenHeader.children("tr").children("td")
					.css(CSS.BORDER_LEFT_COLOR.with(RGBColor.TRANSPARENT));
			frozenHeader.children("tr").children("td")
					.css(CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
			header.children("tr").children("td")
					.css(CSS.BORDER_LEFT_COLOR.with(RGBColor.TRANSPARENT));
			header.children("tr").children("td")
					.css(CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
		}
	}

	private void setData(GQuery frozenBody, GQuery body, GQuery baseFrozenTr,
			GQuery baseTr, Map<Integer, String[]> data, int[] frozenColumns,
			int[] visibleColumns) {
		int height = bodyRowHeight - 1;
		String divClass = enableWordWrapInBodyCells ? "normal" : "nowrap";

		activeRows = data.size() > 0 ? data.get(visibleColumns[0]).length : 0;
		if (enableFrozenTable) {
			frozenBody.empty();
		}
		body.empty();

		for (int row = 0; row < activeRows; row++) {
			String tdId = getId() + ":" + row + ":";
			String trClass = (firstRow + row) % 2 == 0 ? "odd" : "even";
			List<String> values = new ArrayList<String>();

			if (frozenColumns != null && frozenColumns.length > 0) {
				values = new ArrayList<String>();
				for (int column : frozenColumns) {
					values.add(data.get(column)[row]);
				}
				setTrValues(
						createEmptyTr(baseFrozenTr, height, trClass, divClass),
						tdId, values, frozenColumns).appendTo(frozenBody);
			}

			if (visibleColumns != null) {
				values = new ArrayList<String>();
				for (int column : visibleColumns) {
					values.add(data.get(column)[row]);
				}
				setTrValues(createEmptyTr(baseTr, height, trClass, divClass),
						tdId, values, visibleColumns).appendTo(body);
			}
		}
	}

	private void markFilteredColumns(GQuery frozenBody, GQuery body) {
		for (Entry<Integer, List<Object>> filter : columnFilters.entrySet()) {
			String filterId = "td [id$=':" + filter.getKey() + "']";
			if (enableFrozenTable) {
				frozenBody.children("tr").children(filterId).addClass("filter");
			}
			body.children("tr").children(filterId).addClass("filter");
		}
	}

	private void setScrollableContainerWidthAndHeight(int bodyHeight,
			int frozenScrollbarHeight, int scrollbarHeight) {
		// calculate scrollbar's addition to body container height
		int frozenTableHeightOffset = Math.max(0, scrollbarHeight
				- frozenScrollbarHeight);
		int tableHeightOffset = Math.max(0, frozenScrollbarHeight
				- scrollbarHeight);
		frozenTableHeightOffset += frozenTableHeightOffset > 0 ? browserOffset
				: 0;
		tableHeightOffset += tableHeightOffset > 0 ? browserOffset : 0;

		// set frozen body container width and height
		if (enableFrozenTable) {
			GQuery frozenContainer = $("table#dt-frozen-body-table" + getId())
					.parent();
			GQuery frozenContainerLength = $("div#dt-frozen-body-container-length"
					+ getId());
			frozenContainerLength.css(CSS.WIDTH.with(Length.pct(100)));
			frozenContainer.css(CSS.HEIGHT.with(Length.px(this.tableHeight)));
			frozenContainerLength.css(CSS.HEIGHT.with(Length.px(bodyHeight
					+ frozenTableHeightOffset)));
		}

		// set body container width and height
		GQuery container = $("table#dt-body-table" + getId()).parent();
		GQuery containerLength = $("div#dt-body-container-length" + getId());
		GQuery header = $("table#dt-header-table" + getId());
		containerLength.css(CSS.WIDTH.with(allColumnsVisible ? Length.pct(100)
				: Length.px(header.width())));
		container.css(CSS.HEIGHT.with(Length.px(this.tableHeight)));
		containerLength.css(CSS.HEIGHT.with(Length.px(bodyHeight
				+ tableHeightOffset)));
	}

	private void setPageableContainerWidthAndHeight(int bodyHeight,
			int tableHeight) {
		// set frozen body container width and height
		if (enableFrozenTable) {
			GQuery frozenContainer = $("table#dt-frozen-body-table" + getId())
					.parent();
			GQuery frozenContainerLength = $("div#dt-frozen-body-container-length"
					+ getId());
			frozenContainerLength.css(CSS.WIDTH.with(Length.pct(100)));
			frozenContainer.css(CSS.HEIGHT.with(Length.px(tableHeight)));
			frozenContainerLength.css(CSS.HEIGHT.with(Length.pct(100)));
		}

		// set body container width and height
		GQuery container = $("table#dt-body-table" + getId()).parent();
		GQuery containerLength = $("div#dt-body-container-length" + getId());
		GQuery header = $("table#dt-header-table" + getId());
		containerLength.css(CSS.WIDTH.with(allColumnsVisible ? Length.pct(100)
				: Length.px(header.width())));
		container.css(CSS.HEIGHT.with(Length.px(tableHeight)));
		containerLength.css(CSS.HEIGHT.with(Length.pct(100)));
	}

	private void setNonfixedContainerWidthAndHeight() {
		// set frozen body container width and height
		if (enableFrozenTable) {
			GQuery frozenContainer = $("table#dt-frozen-body-table" + getId())
					.parent();
			GQuery frozenContainerLength = $("div#dt-frozen-body-container-length"
					+ getId());
			frozenContainerLength.css(CSS.WIDTH.with(Length.pct(100)));

			frozenContainer.css(CSS.HEIGHT.with(Length.pct(100)));
			frozenContainerLength.css(CSS.HEIGHT.with(Length.pct(100)));
		}

		// set body container width and height
		GQuery container = $("table#dt-body-table" + getId()).parent();
		GQuery containerLength = $("div#dt-body-container-length" + getId());
		GQuery header = $("table#dt-header-table" + getId());
		containerLength.css(CSS.WIDTH.with(allColumnsVisible ? Length.pct(100)
				: Length.px(header.width())));
		container.css(CSS.HEIGHT.with(Length.pct(100)));
		containerLength.css(CSS.HEIGHT.with(Length.pct(100)));
	}

	private void setContainerWidthAndHeight() {
		int frozenScrollbarHeight = 0;
		int scrollbarHeight = 0;

		// calculate horizontal scrollbar height
		if (enableFrozenTable && showFrozenTable) {
			GQuery frozenBody = $("table#dt-frozen-body-table" + getId())
					.parent();
			frozenScrollbarHeight = frozenBody.outerHeight()
					- frozenBody.height();
		}
		GQuery body = $("table#dt-body-table" + getId()).parent();
		scrollbarHeight = body.outerHeight() - body.height();

		int bodyHeight = enablePageableTable ? (this.tableHeight + 1)
				: (bodyRowHeight * allRows);
		int tableHeight = bodyHeight
				+ Math.max(frozenScrollbarHeight, scrollbarHeight);
		if (!enablePageableTable
				&& (enableFixedTableHeight || this.tableHeight <= tableHeight)) {
			setScrollableContainerWidthAndHeight(bodyHeight,
					frozenScrollbarHeight, scrollbarHeight);
		} else if (enablePageableTable && enableFixedTableHeight
				&& pager.getAllPages() > 1) {
			setPageableContainerWidthAndHeight(bodyHeight, tableHeight);
		} else {
			setNonfixedContainerWidthAndHeight();
		}

		// add vertical scrollbar's width to header wrapper (2nd td element)
		int width = body.outerWidth() - body.width();
		$("div#v-dtable" + getId() + " td#dt-header-wrapper-empty").css(
				CSS.WIDTH.with(Length.px(width)));
	}

	private void checkTableSettings(GQuery frozenHeader, GQuery frozenBody,
			GQuery body, int left, int top) {
		// set body container width and height
		setContainerWidthAndHeight();

		if (enableFrozenTable) {
			frozenHeader.parent().scrollLeft(scrollLeftFrozen);
			frozenBody.parent().scrollLeft(scrollLeftFrozen);
			if (BrowserInfo.get().isFirefox()) {
				// FF bug: flickering if vertical scrollbar is hidden
				frozenBody
						.css(CSS.MARGIN_TOP.with(Length.px(-scrollTop + top)));
				ffMarginTop = top;
			} else {
				// set body container position
				frozenBody.css(CSS.TOP.with(Length.px(top)));
				// scroll table body into origin position
				frozenBody.parent().scrollTop(scrollTop);
			}
		}

		// set body container position
		body.css(CSS.LEFT.with(Length.px(left)));
		body.css(CSS.TOP.with(Length.px(top)));

		// scroll table body into origin position
		body.parent().scrollLeft(scrollLeft);
		body.parent().scrollTop(scrollTop);
	}

	private void insertData(Map<Integer, String[]> data, int[] frozenColumns,
			int[] visibleColumns, int left, int top) {
		GQuery frozenHeader = $("table#dt-frozen-header-table" + getId());
		GQuery frozenBody = $("table#dt-frozen-body-table" + getId());
		GQuery body = $("table#dt-body-table" + getId());
		GQuery baseFrozenTr = createBaseTr(frozenBody.parent(), frozenColumns,
				left, false);
		GQuery baseTr = createBaseTr(body.parent(), visibleColumns, left, true);

		setData(frozenBody, body, baseFrozenTr, baseTr, data, frozenColumns,
				visibleColumns);
		markFilteredColumns(frozenBody, body);
		checkTableSettings(frozenHeader, frozenBody, body, left, top);
	}

	@Override
	protected void onLoad() {
		initContainers();

		activateTableResizeEventHandler(true);
		activateTableScrollEventHandler(true);
		activateHeaderCellClickEventHandler(true);
		activateHeaderCellContextMenuEventHandler(true);
		activateBodyCellContextMenuEventHandler(true);
	}

	@Override
	protected void onUnload() {
		activateSplitterClickEventHandler(false);
		activateSplitterMoveEventHandler(false);

		activateFrozenTableScrollEventHandler(false);
		activateFrozenHeaderCellHoverEventHandler(false);
		activateFrozenHeaderCellClickEventHandler(false);
		activateFrozenHeaderCellContextMenuEventHandler(false);
		activateFrozenBodyRowHighlightEventHandler(false);
		activateFrozenBodyCellHoverEventHandler(false);
		activateFrozenBodyCellContextMenuEventHandler(false);

		activateTableResizeEventHandler(false);
		activateTableScrollEventHandler(false);
		activateHeaderCellHoverEventHandler(false);
		activateHeaderCellClickEventHandler(false);
		activateHeaderCellContextMenuEventHandler(false);
		activateBodyRowHighlightEventHandler(false);
		activateBodyCellHoverEventHandler(false);
		activateBodyCellContextMenuEventHandler(false);
	}

	private void sentMessage(final String eventType, final Object eventMessage) {
		if ("newFilter".equals(eventType) || "removeFilter".equals(eventType)
				|| "sortings".equals(eventType) || "freeze".equals(eventType)
				|| "pageReload".equals(eventType)) {
			firstRow = 0;
			scrollTop = 0;
			if (enablePageableTable) {
				pager.reset();
			}
		}

		// standard message elements for page reload
		request.put("prContent", Boolean.toString(showFrozenTable) + ":"
				+ tableFrozenWidth + ":" + scrollLeftFrozen + ":" + scrollLeft
				+ ":" + (enablePageableTable ? 0 : scrollTop) + ":"
				+ (enablePageableTable ? pager.getFirstPage() : 0) + ":"
				+ (enablePageableTable ? pager.getCurentPage() : 0) + ":"
				+ (enablePageableTable ? pager.getAllPages() : 0));

		// standard message elements
		request.put("left", scrollLeft);
		request.put("top", enablePageableTable ? ((pager.getCurentPage() - 1)
				* pageSize * bodyRowHeight) : scrollTop);
		request.put("width", BrowserInfo.get().getScreenWidth());
		request.put("pageSize", enablePageableTable ? pageSize
				: (int) (tableHeight / bodyRowHeight) + 3);

		// non-standard message elements
		request.put(eventType, eventMessage);

		// send request to server
		client.updateVariable(uidl, "request", request, true);
		request.clear();
	}

	protected void sendMessage(final String eventType,
			final Object eventMessage, boolean showLoadingMessage) {
		if (showLoadingMessage && enableLoadingMessage) {
			showLoadingMessage(true);
			$("table#dt-body-table" + getId())
					.parent()
					.parent()
					.parent()
					.as(Effects)
					.animate("opacity:" + loadingMessageOpacity,
							loadingMessageShowDelay, new Function() {
								public void f() {
									sentMessage(eventType, eventMessage);
								}
							});
		} else {
			sentMessage(eventType, eventMessage);
		}
	}

	public void updateFromUIDL(UIDL uidl, ApplicationConnection client) {
		if (client.updateComponent(this, uidl, true)) {
			return;
		}

		this.client = client;
		this.uidl = uidl.getId();
		this.request.clear();

		// START: this must be checked first
		if (uidl.hasAttribute("enablePageableTable")) {
			enablePageableTable = uidl
					.getBooleanAttribute("enablePageableTable");
			if (enablePageableTable) {
				pager = new VPager(this);
				getElement().appendChild(pager.getElement());
				pager.show(false);
			}
		}

		if (uidl.hasAttribute("enableFixedTableHeight")) {
			enableFixedTableHeight = uidl
					.getBooleanAttribute("enableFixedTableHeight");
		}

		if (uidl.hasAttribute("enableAllRows") && !enablePageableTable) {
			enableAllRows = uidl.getBooleanAttribute("enableAllRows");
		}

		if (uidl.hasAttribute("enableAllColumns")) {
			enableAllColumns = uidl.getBooleanAttribute("enableAllColumns");
		}

		if (uidl.hasAttribute("enableColumnNumberInWaitBox")) {
			// table.showColumnNumbersInWaitBox(uidl
			// .getBooleanAttribute("enableColumnNumberInWaitBox"));
		}

		if (uidl.hasAttribute("enableFrozenTable")) {
			enableFrozenTable = uidl.getBooleanAttribute("enableFrozenTable");
			showFrozenTable = enableFrozenTable;
			showSplitter = enableFrozenTable;

			if (enableFrozenTable) {
				appendFrozenTable();
				activateFrozenTableScrollEventHandler(true);
				activateFrozenHeaderCellClickEventHandler(true);
				activateFrozenHeaderCellContextMenuEventHandler(true);
				activateFrozenBodyCellContextMenuEventHandler(true);
				activateTableScrollEventHandler(true);
			}
		}

		if (uidl.hasAttribute("enableHideFrozenTable")) {
			enableHideFrozenTable = uidl
					.getBooleanAttribute("enableHideFrozenTable");
			if (enableFrozenTable && enableHideFrozenTable
					&& splitterClickEventHandler == null) {
				activateSplitterClickEventHandler(true);
				$("#dt-splitter-container").css(
						CSS.CURSOR.with(Cursor.COL_RESIZE));
			}
		}

		if (uidl.hasAttribute("enableResizeFrozenTable")) {
			enableResizeFrozenTable = uidl
					.getBooleanAttribute("enableResizeFrozenTable");
			if (enableFrozenTable && enableResizeFrozenTable
					&& splitterClickEventHandler == null) {
				activateSplitterClickEventHandler(true);
				$("#dt-splitter-container").css(
						CSS.CURSOR.with(Cursor.COL_RESIZE));
			}
		}

		if (uidl.hasAttribute("enableEmptyFrozenTable")) {
			enableEmptyFrozenTable = uidl
					.getBooleanAttribute("enableEmptyFrozenTable");
		}

		if (uidl.hasAttribute("enableSortedFrozenTable") && enableFrozenTable) {
			enableSortedFrozenTable = uidl
					.getBooleanAttribute("enableSortedFrozenTable");
		}

		if (uidl.hasAttribute("enableSortedTable")) {
			enableSortedTable = uidl.getBooleanAttribute("enableSortedTable");
		}

		if (uidl.hasAttribute("enableFilteredFrozenTable") && enableFrozenTable) {
			enableFilteredFrozenTable = uidl
					.getBooleanAttribute("enableFilteredFrozenTable");
		}

		if (uidl.hasAttribute("enableFilteredTable")) {
			enableFilteredTable = uidl
					.getBooleanAttribute("enableFilteredTable");
		}

		if (uidl.hasAttribute("enableFilterHelper")) {
			enableFilterHelper = uidl.getBooleanAttribute("enableFilterHelper");
		}

		if (uidl.hasAttribute("enableFilterHelperOnTop")) {
			enableFilterHelperOnTop = uidl
					.getBooleanAttribute("enableFilterHelperOnTop");
		}

		if (uidl.hasAttribute("enableFreezeAnyColumns")) {
			enableFreezeAnyColumns = uidl
					.getBooleanAttribute("enableFreezeAnyColumns");
			if (enableFreezeAnyColumns
					&& headerCellContextMenuEventHandler == null) {
				activateHeaderCellContextMenuEventHandler(true);
				if (enableFrozenTable
						&& frozenHeaderCellContextMenuEventHandler == null) {
					activateFrozenHeaderCellContextMenuEventHandler(true);
				}
			}
		}

		if (uidl.hasAttribute("enableUnfreezeAnyColumns")) {
			enableUnfreezeAnyColumns = uidl
					.getBooleanAttribute("enableUnfreezeAnyColumns");
			if (enableUnfreezeAnyColumns
					&& headerCellContextMenuEventHandler == null) {
				activateHeaderCellContextMenuEventHandler(true);
				if (enableFrozenTable
						&& frozenHeaderCellContextMenuEventHandler == null) {
					activateFrozenHeaderCellContextMenuEventHandler(true);
				}
			}
		}

		if (uidl.hasAttribute("enableHighlightActiveRow")) {
			boolean enable = uidl
					.getBooleanAttribute("enableHighlightActiveRow");
			activateBodyRowHighlightEventHandler(enable);
			if (enableFrozenTable) {
				activateFrozenBodyRowHighlightEventHandler(enable);
			}
		}

		if (uidl.hasAttribute("enableWordWrapInHeaderCells")) {
			enableWordWrapInHeaderCells = uidl
					.getBooleanAttribute("enableWordWrapInHeaderCells");
			if (enableWordWrapInHeaderCells) {
				activateHeaderCellHoverEventHandler(false);
			}
			if (enableFrozenTable && enableWordWrapInHeaderCells) {
				activateFrozenHeaderCellHoverEventHandler(false);
			}
		}

		if (uidl.hasAttribute("enableWordWrapInBodyCells")) {
			enableWordWrapInBodyCells = uidl
					.getBooleanAttribute("enableWordWrapInBodyCells");
			if (enableWordWrapInBodyCells) {
				activateBodyCellHoverEventHandler(false);
			}
			if (enableFrozenTable && enableWordWrapInBodyCells) {
				activateFrozenBodyCellHoverEventHandler(false);
			}
		}

		if (uidl.hasAttribute("enableExpandedFrozenHeaderCell")
				&& enableFrozenTable) {
			enableExpandedFrozenHeaderCell = uidl
					.getBooleanAttribute("enableExpandedFrozenHeaderCell");
			activateFrozenHeaderCellHoverEventHandler(enableExpandedFrozenHeaderCell);
		}

		if (uidl.hasAttribute("enableExpandedFrozenHeaderCellForActiveColumn")) {
			enableExpandedFrozenHeaderCellForActiveColumn = uidl
					.getBooleanAttribute("enableExpandedFrozenHeaderCellForActiveColumn");
		}

		if (uidl.hasAttribute("enableExpandedFrozenBodyCell")
				&& enableFrozenTable) {
			enableExpandedFrozenBodyCell = uidl
					.getBooleanAttribute("enableExpandedFrozenBodyCell");
			activateFrozenBodyCellHoverEventHandler(enableExpandedFrozenBodyCell);
		}

		if (uidl.hasAttribute("enableExpandedHeaderCell")) {
			enableExpandedHeaderCell = uidl
					.getBooleanAttribute("enableExpandedHeaderCell");
			activateHeaderCellHoverEventHandler(enableExpandedHeaderCell);
		}

		if (uidl.hasAttribute("enableExpandedHeaderCellForActiveColumn")) {
			enableExpandedHeaderCellForActiveColumn = uidl
					.getBooleanAttribute("enableExpandedHeaderCellForActiveColumn");
		}

		if (uidl.hasAttribute("enableExpandedBodyCell")) {
			enableExpandedBodyCell = uidl
					.getBooleanAttribute("enableExpandedBodyCell");
			activateBodyCellHoverEventHandler(enableExpandedBodyCell);
		}

		if (uidl.hasAttribute("enableHorizontalLines")) {
			enableHorizontalLines = uidl
					.getBooleanAttribute("enableHorizontalLines");
		}

		if (uidl.hasAttribute("enableVerticalLines")) {
			enableVerticalLines = uidl
					.getBooleanAttribute("enableVerticalLines");
		}

		if (uidl.hasAttribute("enableLoadingMessage")) {
			enableLoadingMessage = uidl
					.getBooleanAttribute("enableLoadingMessage");
			appendLoadingMessageBox(enableLoadingMessage);
		}
		// END: this must be checked first

		// START: this must be checked second
		if (uidl.hasAttribute("loadingMessageOpacity")) {
			loadingMessageOpacity = uidl
					.getFloatAttribute("loadingMessageOpacity");
		}

		if (uidl.hasAttribute("loadingMessage")) {
			loadingMessage = uidl.getStringAttribute("loadingMessage");
			appendLoadingMessageBox(enableLoadingMessage);
		}

		if (uidl.hasAttribute("loadingMessageShowDelay")
				&& uidl.hasAttribute("loadingMessageHideDelay")) {
			loadingMessageShowDelay = uidl
					.getIntAttribute("loadingMessageShowDelay");
			loadingMessageHideDelay = uidl
					.getIntAttribute("loadingMessageHideDelay");
			appendLoadingMessageBox(enableLoadingMessage);
		}

		if (uidl.hasAttribute("emptyDataMessage")) {
			emptyDataMessage = uidl.getStringAttribute("emptyDataMessage");
		}

		if (uidl.hasAttribute("allColumnsVisible")) {
			allColumnsVisible = uidl.getBooleanAttribute("allColumnsVisible");
			$("table#dt-header-table" + getId()).css("width",
					allColumnsVisible ? "100%" : "0");
			$("table#dt-body-table" + getId()).css("width",
					allColumnsVisible ? "100%" : "0");
		}

		if (uidl.hasAttribute("allRowsVisible")) {
			allRowsVisible = uidl.getBooleanAttribute("allRowsVisible");
		}

		if (uidl.hasAttribute("row")) {
			firstRow = enablePageableTable ? 0 : uidl.getIntAttribute("row");
		}

		if (uidl.hasAttribute("tableFrozenMinimalWidth")) {
			tableFrozenMinimalWidth = uidl
					.getIntAttribute("tableFrozenMinimalWidth");
			tableFrozenWidth = tableFrozenWidth < tableFrozenMinimalWidth ? tableFrozenMinimalWidth
					: tableFrozenWidth;
			tableFrozenWidth = enableFrozenTable ? tableFrozenWidth : 0;
		}

		if (uidl.hasAttribute("pageSize")) {
			pageSize = uidl.getIntAttribute("pageSize");
		}

		if (uidl.hasAttribute("allRows")) {
			allRows = uidl.getIntAttribute("allRows");
			if (enablePageableTable && pager != null) {
				pager.setAllPages(allRows / pageSize
						+ (allRows % pageSize > 0 ? 1 : 0));
				// pager.reset();
			}
		}

		if (uidl.hasAttribute("pagerTrayPages") && pager != null) {
			pager.setTrayPages(uidl.getIntAttribute("pagerTrayPages"));
			pager.setAllPages(allRows / pageSize
					+ (allRows % pageSize > 0 ? 1 : 0));
		}

		if (uidl.hasAttribute("headerRowHeight")) {
			headerRowHeight = uidl.getIntAttribute("headerRowHeight");
		}

		if (uidl.hasAttribute("bodyRowHeight")) {
			bodyRowHeight = uidl.getIntAttribute("bodyRowHeight");
		}

		if (uidl.hasAttribute("tableMinimalWidth")) {
			tableMinimalWidth = uidl.getIntAttribute("tableMinimalWidth");
		}

		if (uidl.hasAttribute("tableHeight")) {
			tableHeight = enablePageableTable ? bodyRowHeight * pageSize : uidl
					.getIntAttribute("tableHeight") - headerRowHeight;
		}

		if (uidl.hasAttribute("hiddenColumns")) {
			int[] columns = uidl.getIntArrayAttribute("hiddenColumns");
			Arrays.sort(columns);
			columnHiddens.clear();
			for (int column : columns) {
				columnHiddens.add(column);
			}
		}

		if (uidl.hasAttribute("formatedColumns")) {
			int[] columns = uidl.getIntArrayAttribute("formatedColumns");
			Arrays.sort(columns);
			columnFormatted.clear();
			for (int column : columns) {
				columnFormatted.add(column);
			}
		}

		if (uidl.hasAttribute("enableFreezeColumns")) {
			int[] columns = uidl.getIntArrayAttribute("enableFreezeColumns");
			columnFreezeEnabled.clear();
			for (int column : columns) {
				columnFreezeEnabled.add(column);
			}
		}

		if (uidl.hasAttribute("enableUnfreezeColumns")) {
			int[] columns = uidl.getIntArrayAttribute("enableUnfreezeColumns");
			columnUnfreezeEnabled.clear();
			for (int column : columns) {
				columnUnfreezeEnabled.add(column);
			}
		}

		if (uidl.hasAttribute("newFilter")) {
			String[] filter = uidl.getStringArrayAttribute("newFilter");
			setFilter(Integer.parseInt(filter[0]), filter[1], filter[2], true);
		}

		if (uidl.hasAttribute("removeFilter")) {
			setFilter(uidl.getIntAttribute("removeFilter"), null, null, false);
		}

		if (uidl.hasAttribute("columnWidths")) {
			columnWidths.clear();
			int[] widths = uidl.getIntArrayAttribute("columnWidths");
			for (int column = 0; column < widths.length; column++) {
				columnWidths.add(widths[column]);
			}
		}

		if (uidl.hasAttribute("columnAlignments")) {
			columnAlignments.clear();
			for (int alignment : uidl.getIntArrayAttribute("columnAlignments")) {
				columnAlignments.add(alignment);
			}
		}

		if (uidl.hasAttribute("columnHeaders")
				&& uidl.hasAttribute("headerAligments")) {
			setHeader(uidl.getStringArrayAttribute("columnHeaders"),
					uidl.getIntArrayAttribute("headerAligments"),
					showFrozenTable ? tableFrozenWidth : 0);
			columnFrozens.clear();
		}
		// END: this must be checked second

		if (uidl.hasAttribute("noData")) {
			showLoadingMessage(false);
			sendMessage("noData", "noData", false);
		} else if (uidl.hasAttribute("emptyData")) {
			getElement().setInnerHTML(emptyDataMessage);
		} else if (uidl.hasAttribute("visibleColumns")
				&& uidl.hasAttribute("left") && uidl.hasAttribute("top")) {
			Map<Integer, String[]> data = new HashMap<Integer, String[]>();

			int frozenSize = 0;
			int[] frozenColumns = null;
			if (enableFrozenTable) {
				if (uidl.hasAttribute("frozenColumns")) {
					frozenColumns = uidl.getIntArrayAttribute("frozenColumns");
					if (columnFrozens.size() != frozenColumns.length) {
						setFrozenHeader(frozenColumns);
					}
					for (int column : frozenColumns) {
						if (uidl.hasAttribute("ff" + column)) {
							data.put(column,
									uidl.getStringArrayAttribute("ff" + column));
						}
					}
					frozenSize = frozenColumns.length;

					showSplitter(true);
					showFrozenTable(showFrozenTable);
					if (splitterClickEventHandler == null) {
						activateSplitterClickEventHandler(showFrozenTable);
					}
				} else {
					showSplitter(false);
					showFrozenTable(false);
					activateSplitterClickEventHandler(false);
				}
			}

			int visibleSize = 0;
			int[] visibleColumns = null;
			if (uidl.hasAttribute("visibleColumns")) {
				visibleColumns = uidl.getIntArrayAttribute("visibleColumns");
				for (int column : visibleColumns) {
					if (uidl.hasAttribute("vv" + column)) {
						data.put(column,
								uidl.getStringArrayAttribute("vv" + column));
					}
				}
				visibleSize = visibleColumns.length;
			}

			if (data.size() == (frozenSize + visibleSize)) {
				if (uidl.hasAttribute("pageReload")) {
					String[] prReloadContent = uidl.getStringAttribute(
							"pageReload").split(":");
					showFrozenTable = !Boolean.parseBoolean(prReloadContent[0]);
					tableFrozenWidth = Integer.parseInt(prReloadContent[1]);
					scrollLeftFrozen = Integer.parseInt(prReloadContent[2]);
					scrollLeft = Integer.parseInt(prReloadContent[3]);
					scrollTop = Integer.parseInt(prReloadContent[4]);
					if (enableFrozenTable) {
						showFrozenTable(!showFrozenTable);
					}
					if (enablePageableTable) {
						pager.setAllPages(Integer.parseInt(prReloadContent[5]),
								Integer.parseInt(prReloadContent[6]),
								Integer.parseInt(prReloadContent[7]));
					}
				}

				if (uidl.hasAttribute("sortings")) {
					for (String sorting : uidl
							.getStringArrayAttribute("sortings")) {
						int column = Integer.parseInt(sorting.substring(1));
						setColumnSorting(column,
								sorting.startsWith("+") ? "asc" : "desc");
					}
				}

				int left = uidl.getIntAttribute("left");
				int top = enablePageableTable ? 0 : uidl.getIntAttribute("top");
				insertData(data, frozenColumns, visibleColumns, left, top);

				setContainerWidthAndHeight();
				if (enablePageableTable) {
					pager.show(pager.getAllPages() > 1);
				}
				if (enableLoadingMessage) {
					showLoadingMessage(false);
				}

				// send base table settings to server
				if (informServer) {
					sendMessage("newData", "informServer", false);
					informServer = false;
				}

				// page refresh: this have to be called last because
				// the table context have to be displayed first
				if (uidl.hasAttribute("pageRefresh")) {
					(new Timer() {
						public void run() {
							sendMessage("newData", Boolean.FALSE.toString(),
									false);
						}
					}).schedule(uidl.getIntAttribute("pageRefresh"));
				}
			}
		}
	}

}
