/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.ui;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.vaadin.terminal.gwt.client.ui.VOverlay;

public class VMenu extends VOverlay {

	private int column = 0;
	private int row = 0;
	private VDTable dataTable = null;
	private Element menu = null;
	private Element targetE = null;

	public VMenu(VDTable dataTable) {
		this.dataTable = dataTable;

		getElement().setClassName("dt-context-menu");
		setAutoHideEnabled(true);
		setShadowEnabled(true);

		if (DOM.getChildCount(getElement()) > 0) {
			getElement().getFirstChildElement().removeFromParent();
		}

		sinkEvents(Event.ONCLICK | Event.ONCONTEXTMENU);
	}

	protected void add(Element e) {
		if (menu != null) {
			menu.removeFromParent();
		}

		targetE = e;
		menu = DOM.createDiv();
		getElement().appendChild(menu);

		String[] id = e.getId().split(":");
		String cellValue = $(e).children("div").children("span").html().trim();
		Element item = null;
		List<VMenuItems> menuItems = new ArrayList<VMenuItems>();

		// table header
		if (id.length == 2) {
			column = Integer.parseInt(id[1]);

			// ascending/descending/remove sorting
			if (dataTable.isSortingsEnabled(column)) {
				String sortingType = VMenuItems.SORTING_REMOVE.id();
				if ($(e).hasClass("asc")) {
					sortingType = VMenuItems.SORTING_ASCENDING.id();
				} else if ($(e).hasClass("desc")) {
					sortingType = VMenuItems.SORTING_DESCENDING.id();
				}

				menuItems.addAll(Arrays.asList(VMenuItems.SORTING_ASCENDING,
						VMenuItems.SORTING_DESCENDING,
						VMenuItems.SORTING_REMOVE));
				for (VMenuItems menuItem : menuItems) {
					if (!menuItem.id().equalsIgnoreCase(sortingType)) {
						item = DOM.createDiv();
						item.setId(menuItem.id());
						item.setInnerText(menuItem.description().replace("_",
								cellValue));
						menu.appendChild(item);
					}
				}

				if (($(e).parent().children("td.asc", "td.desc").length() == 1
						&& !$(e).hasClass("asc") && !$(e).hasClass("desc"))
						|| $(e).parent().children("td.asc", "td.desc").length() > 1) {
					item = DOM.createDiv();
					item.setId(VMenuItems.SORTING_REMOVE_ALL.id());
					item.setInnerText(VMenuItems.SORTING_REMOVE_ALL
							.description());
					menu.appendChild(item);
				}
			}

			// freeze/unfreeze column
			if (dataTable.isFrozenTableEnabled()) {
				item = DOM.createDiv();
				item.setId(!dataTable.isFrozen(column) ? VMenuItems.FREEZE.id()
						: VMenuItems.UNFREEZE.id());
				item.setInnerText((!dataTable.isFrozen(column) ? VMenuItems.FREEZE
						.description() : VMenuItems.UNFREEZE.description())
						.replace("_", cellValue));

				if (!dataTable.isFrozen(column)
						&& dataTable.isFreezeAnyColumnsEnabled()
						&& dataTable.isFreezeEnabled(column)
						&& dataTable.getAllColumns(false) > 1) {
					item.setInnerText(VMenuItems.FREEZE.description().replace(
							"_", cellValue));
					menu.appendChild(item);
				} else if (dataTable.isFrozen(column)
						&& dataTable.isUnfreezeEnabled(column)
						&& (dataTable.isFrozenTableEmptyEnabled() || (!dataTable
								.isFrozenTableEmptyEnabled() && dataTable
								.getAllColumns(true) > 1))) {
					item.setInnerText(VMenuItems.UNFREEZE.description()
							.replace("_", cellValue));
					menu.appendChild(item);
				}
			}
		}

		// table body
		else if (id.length == 3) {
			column = Integer.parseInt(id[2]);
			row = Integer.parseInt(id[1]);

			if (dataTable.isFilterEnabled(column)) {
				if (cellValue != null && cellValue.length() > 0
						&& !"&nbsp;".equals(cellValue)) {
					if (dataTable.isFormatted(column)) {
						menuItems.addAll(Arrays.asList(VMenuItems.EQUAL,
								VMenuItems.NOT_EQUAL, VMenuItems.GREATER,
								VMenuItems.GREATER_EQUAL, VMenuItems.LESS,
								VMenuItems.LESS_EQUAL));
					} else {
						menuItems.addAll(Arrays.asList(VMenuItems.LIKE,
								VMenuItems.NOT_LIKE, VMenuItems.LIKE_LEFT,
								VMenuItems.LIKE_RIGHT,
								VMenuItems.LIKE_LEFT_RIGHT));
					}
				}

				if (!dataTable.isFiltered(column)) {
					menuItems.addAll(Arrays.asList(VMenuItems.NULL,
							VMenuItems.NOT_NULL));
				}

				for (VMenuItems menuItem : menuItems) {
					if (!dataTable.filterOperatorExists(column,
							VMenuItems.EQUAL.description())
							&& !dataTable.filterOperatorExists(column,
									VMenuItems.LIKE.description())
							&& !dataTable.filterOperatorExists(column,
									VMenuItems.NULL.description())) {
						String filter = menuItem.description().replace("_",
								cellValue);

						if (menuItem.description().equals(
								VMenuItems.NULL.description())) {
							if (!dataTable.filterExists(column,
									VMenuItems.NOT_NULL.description())) {
								item = DOM.createDiv();
								item.setId(menuItem.id());
								item.setInnerText(filter);
								menu.appendChild(item);
							}
						} else if (!menuItem.description().equals(
								VMenuItems.LIKE_LEFT_RIGHT.description())) {
							if (!dataTable.filterExists(column, filter)) {
								item = DOM.createDiv();
								item.setId(menuItem.id());
								item.setInnerText(filter);
								menu.appendChild(item);
							}
						} else {
							for (int i = 1; i <= 5 && i <= cellValue.length(); i++) {
								filter = menuItem.description().replace("_",
										cellValue.substring(0, i));
								if (!dataTable.filterExists(column, filter)) {
									item = DOM.createDiv();
									item.setId(menuItem.id() + i);
									item.setInnerText(filter);
									menu.appendChild(item);
								}
							}
						}
					}
				}
			}

			if (dataTable.isFiltered(column)) {
				String headerId = "table#dt-"
						+ (dataTable.isFrozen(column) ? "frozen-" : "")
						+ "header-table" + id[0] + " tr";
				String header = $(headerId).children(
						"td [id='" + id[0] + ":" + id[2] + "']").text();
				item = DOM.createDiv();
				item.setId(VMenuItems.FILTER_REMOVE.id());
				item.setInnerText(VMenuItems.FILTER_REMOVE.description()
						.replace("_", header));
				menu.appendChild(item);
			}
		}
	}

	@Override
	public void onBrowserEvent(Event event) {
		event.stopPropagation();

		switch (DOM.eventGetType(event)) {
		case Event.ONCLICK:
			hide();
			String menuItemId = DOM.eventGetTarget(event).getId();
			String menuItem = menuItemId.startsWith(VMenuItems.LIKE_LEFT_RIGHT
					.id()) ? VMenuItems.LIKE_LEFT_RIGHT.id() : menuItemId;

			switch (VMenuItems.item(menuItem)) {
			case SORTING_ASCENDING:
				$(targetE).removeClass("desc").addClass("asc");
				dataTable
						.sendMessage("sortings", dataTable.getSortings(), true);
				break;

			case SORTING_DESCENDING:
				$(targetE).removeClass("asc").addClass("desc");
				dataTable
						.sendMessage("sortings", dataTable.getSortings(), true);
				break;

			case SORTING_REMOVE:
				$(targetE).removeClass("asc", "desc");
				dataTable
						.sendMessage("sortings", dataTable.getSortings(), true);
				break;

			case SORTING_REMOVE_ALL:
				$(targetE).parent().children("td.asc", "td.desc")
						.removeClass("asc", "desc");
				dataTable
						.sendMessage("sortings", dataTable.getSortings(), true);
				break;

			case EQUAL:
			case NOT_EQUAL:
			case GREATER:
			case GREATER_EQUAL:
			case LESS:
			case LESS_EQUAL:
			case LIKE:
			case NOT_LIKE:
			case LIKE_LEFT:
			case LIKE_RIGHT:
			case LIKE_LEFT_RIGHT:
				// column, row, operator, description, filter
				dataTable.newFilter(column, row, VMenuItems.item(menuItem)
						.operator(), VMenuItems.item(menuItem).description(),
						DOM.eventGetTarget(event).getInnerText());
				break;

			case NULL:
			case NOT_NULL:
				// column, row, operator, description, filter
				dataTable.newFilter(column, row, VMenuItems.item(menuItem)
						.operator(), VMenuItems.item(menuItem).description(),
						DOM.eventGetTarget(event).getInnerText());
				break;

			case FREEZE:
			case UNFREEZE:
				dataTable.setFrozen(column);
				break;

			case FILTER_REMOVE:
				dataTable.removeFilters(column);
				break;

			case NONE:
			default:
				break;
			}
			break;

		case Event.ONCONTEXTMENU:
			event.preventDefault();
			break;

		default:
			break;
		}
	}

	@Override
	public void show() {
		if (menu.getChildCount() > 0) {
			super.show();

			int popupLeft = (getAbsoluteLeft() + getOffsetWidth() + 5) <= Window
					.getClientWidth() ? getPopupLeft() : (Window
					.getClientWidth() - getOffsetWidth() - 5);
			int popupTop = (getAbsoluteTop() + getOffsetHeight() + 5) <= Window
					.getClientHeight() ? getPopupTop() : (Window
					.getClientHeight() - getOffsetHeight() - 5);

			setPopupPosition(popupLeft, popupTop);
		}
	}

}
