/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.ui;

import static com.google.gwt.query.client.GQuery.$;

import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.ui.SimplePanel;

public class VPager extends SimplePanel {

	private int allPages = 0;
	private int curentPage = 1;
	private int firstPage = 1;
	private int lastPage = 5;
	private int trayPages = lastPage;

	private VDTable dTable = null;
	private GQuery pagerClickEventHandler;

	public VPager(VDTable dTable) {
		setStyleName("dt-pager");
		getElement().setId("dt-pager");
		this.dTable = dTable;
	}

	private void activatePagerClickEventHandler(boolean enable) {
		if (enable) {
			GQuery pagerDiv = $("div#v-dtable" + dTable.getId()
					+ " div#dt-pager");

			activatePagerClickEventHandler(false);
			pagerClickEventHandler = pagerDiv.delegate("li", "click",
					new Function() {
						public void f(Element e) {
							if ("fp".equals(e.getId())) {
								// first page
								setCurentPage(1);
							} else if ("pps".equals(e.getId())) {
								// previous pages
								setCurentPage(getCurentPage() - getTrayPages());
							} else if ("pp".equals(e.getId())) {
								// previous page
								setCurentPage(getCurentPage() - 1);
							} else if ("np".equals(e.getId())) {
								// next page
								setCurentPage(getCurentPage() + 1);
							} else if ("nps".equals(e.getId())) {
								// next pages
								setCurentPage(getCurentPage() + getTrayPages());
							} else if ("lp".equals(e.getId())) {
								// last page
								setCurentPage(allPages);
							} else if (e.getId() != null
									&& e.getId().startsWith("page")) {
								// current num page
								setCurentPage(Integer.parseInt(e.getId()
										.substring("page".length())));
							}
							checkButtons(true);
						}
					});
		} else if (!enable && pagerClickEventHandler != null) {
			pagerClickEventHandler.undelegate("li", "click");
			pagerClickEventHandler = null;
		}
	}

	private void checkButtons(boolean sendMessage) {
		if (getCurentPage() < getFirstPage()) {
			setFirstPage(getCurentPage());
			setLastPage(getFirstPage() + getTrayPages() - 1);
			createNavigationButtons(getFirstPage(), getLastPage(), sendMessage);
		} else if (getCurentPage() > getLastPage()) {
			setLastPage(getCurentPage());
			setFirstPage(getCurentPage() - getTrayPages() + 1);
			createNavigationButtons(getFirstPage(), getLastPage(), sendMessage);
		} else {
			activateNumberButton(getCurentPage(), sendMessage);
			activatePrevoiusButtons(getCurentPage() > 1);
			activateNextButtons(getCurentPage() < allPages);
		}
	}

	protected void show(boolean show) {
		if (!show && isVisible()) {
			getElement().getStyle().setDisplay(Display.NONE);
			activatePagerClickEventHandler(false);
		} else if (show && !isVisible()) {
			getElement().getStyle().clearDisplay();
			activatePagerClickEventHandler(true);
		}
	}

	private void createNavigationButtons(int firstPage, int lastPage,
			boolean sendMessage) {
		GQuery buttonsBox = $("<ul/>");

		// page navigation: previous
		if (!(allPages < trayPages)) {
			$("<li/>").id("fp").text("∣≪").addClass("char-active")
					.appendTo(buttonsBox);
		}
		$("<li/>").addClass("char-space").appendTo(buttonsBox);
		$("<li/>").id("pps").text("≪").addClass("char-active")
				.appendTo(buttonsBox);
		$("<li/>").addClass("char-space").appendTo(buttonsBox);
		$("<li/>").id("pp").text("<").addClass("char-active")
				.appendTo(buttonsBox);

		// page navigation: nums
		for (int page = firstPage; page <= lastPage; page++) {
			$("<li/>").id("page" + page).text("" + page).addClass("num")
					.appendTo(buttonsBox);
		}

		// page navigation: next
		$("<li/>").id("np").text(">").addClass("char-active")
				.appendTo(buttonsBox);
		$("<li/>").addClass("char-space").appendTo(buttonsBox);
		$("<li/>").id("nps").text("≫").addClass("char-active")
				.appendTo(buttonsBox);
		$("<li/>").addClass("char-space").appendTo(buttonsBox);
		if (!(allPages < trayPages)) {
			$("<li/>").id("lp").text("≫∣").addClass("char-active")
					.appendTo(buttonsBox);
		}

		$(getElement()).empty().append(buttonsBox);

		activateNumberButton(getCurentPage(), sendMessage);
		activatePrevoiusButtons(getCurentPage() > 1);
		activateNextButtons(getCurentPage() < allPages);
	}

	protected void reset() {
		setAllPages(allPages);
		checkButtons(false);
	}

	protected int getAllPages() {
		return allPages;
	}

	protected void setAllPages(int firstPage, int currentPage, int allPages) {
		this.allPages = allPages;
		if (allPages > 0) {
			setFirstPage(firstPage);
			setLastPage(getFirstPage() + getTrayPages() - 1);
			setCurentPage(currentPage);
			createNavigationButtons(getFirstPage(), getLastPage(), false);
		}
	}

	protected void setAllPages(int allPages) {
		this.allPages = allPages;
		if (allPages > 0) {
			setFirstPage(1);
			setLastPage(getFirstPage() + getTrayPages() - 1);
			setCurentPage(1);
			createNavigationButtons(getFirstPage(), getLastPage(), false);
		}
	}

	private int getTrayPages() {
		return (trayPages > allPages ? allPages : trayPages);
	}

	protected void setTrayPages(int trayPages) {
		this.trayPages = trayPages <= 0 ? 5 : trayPages;
	}

	protected int getFirstPage() {
		return firstPage;
	}

	private void setFirstPage(int firstPage) {
		this.firstPage = firstPage < 1 ? 1 : firstPage;
	}

	private int getLastPage() {
		return lastPage;
	}

	private void setLastPage(int lastPage) {
		this.lastPage = lastPage > allPages ? allPages : lastPage;
	}

	protected int getCurentPage() {
		return curentPage;
	}

	private void setCurentPage(int curentPage) {
		this.curentPage = curentPage < 1 ? 1
				: (curentPage > allPages ? allPages : curentPage);
	}

	private void activatePrevoiusButtons(boolean activate) {
		$("div#v-dtable" + dTable.getId() + " div#dt-pager ul")
				.children("li#fp", "li#pps", "li#pp")
				.removeClass("char-active", "char")
				.addClass(activate ? "char-active" : "char");
	}

	private void activateNextButtons(boolean activate) {
		$("div#v-dtable" + dTable.getId() + " div#dt-pager ul")
				.children("li#np", "li#nps", "li#lp")
				.removeClass("char-active", "char")
				.addClass(activate ? "char-active" : "char");
	}

	private void activateNumberButton(int currentPage, boolean sendMessage) {
		boolean isActive = $(
				"div#v-dtable" + dTable.getId() + " div#dt-pager ul").children(
				"li#page" + currentPage).hasClass("num-active");

		$("div#v-dtable" + dTable.getId() + " div#dt-pager ul").children(
				"li.num-active").toggleClass("num-active", "num");
		$("div#v-dtable" + dTable.getId() + " div#dt-pager ul").children(
				"li#page" + currentPage).toggleClass("num-active", "num");

		if (!isActive && sendMessage) {
			dTable.sendMessage("newData", Boolean.TRUE.toString(), true);
		}
	}

}
